@@ -15,6 +15,11 @@
 
 #if defined(SH_USE_KERN) 
 
+#undef _
+#define _(string)  string
+#undef N_
+#define N_(string) string
+
 void usage(int flag)
 {
   printf("\n");
@@ -40,6 +45,7 @@ void usage(int flag)
 #include <errno.h>
 #include <limits.h>
 #include <sys/utsname.h>
+#include <sys/mman.h>
 
 /* number of system calls */
 #define SH_MAXCALLS 512
@@ -66,6 +72,63 @@ union {
 static unsigned long addr_system_call;
 static unsigned char system_call_code[SYS_CODE_SIZE];
 
+static int kmem_read (int fd, unsigned long addr, unsigned char * buf, int len)
+{
+  if (lseek(fd, addr, SEEK_SET) == (off_t) (-1))
+    {
+      if (verbose)
+	perror("kmem_read: lseek");
+      return -1;
+    }
+  if (read(fd, buf, len) < 0)
+    {
+      if (verbose)
+	perror("kmem_read: read");
+      return -1;
+    }
+  return 0;
+}
+
+static int kmem_mmap (int fd, unsigned long addr, unsigned char * buf, int len)
+{
+  size_t    moff, roff;
+  size_t    sz;
+  char    * kmap;
+
+
+  /* first, try read()
+   */
+  int  ret_old = kmem_read (fd, addr, buf, len); 
+
+  if (ret_old == 0)
+    return ret_old;
+
+  if (verbose)
+    fprintf(stderr, 
+	    "kmem_mmap: read() from /dev/kmem failed, now trying mmap()\n");
+
+  sz = getpagesize(); /* unistd.h */
+
+  moff = ((size_t)(addr/sz)) * sz;                 /* lower page boundary */
+  roff = addr - moff;    /* off relative to lower address of mmapped area */
+  kmap = mmap(0, len+sz, PROT_READ, MAP_PRIVATE, fd, moff);/* sys/mman.h */
+
+  if (kmap == MAP_FAILED)
+    {
+      perror("kmem_mmap: mmap");
+      return -1;
+    }
+  memcpy (buf, &kmap[roff], len);
+      
+  if (munmap(kmap, len+sz) != 0)
+    {
+      perror("kmem_mmap: munmap");
+      return -1;
+    }
+
+  return 0;
+}
+
 int read_kcode (unsigned long addr, unsigned char * buf, int len)
 {
   int fd;
@@ -79,19 +142,15 @@ int read_kcode (unsigned long addr, unsigned char * buf, int len)
   fd = open ("/dev/kmem", O_RDONLY);
   if (fd < 0)
     {
-      perror("read_kcode: open");
+      perror("read_kcode: open /dev/kmem");
       return -1;
     }
-  if (lseek(fd, addr, SEEK_SET) == (off_t) (-1))
+  if (kmem_mmap(fd, addr, buf, len) < 0)
     {
-      perror("read_kcode: lseek");
-      return -1;
-    }
-  if (read(fd, buf, len) < 0)
-    {
-      perror("read_kcode: read");
+      close (fd);
       return -1;
     }
+  close (fd);
   return 0;
 }
 
@@ -294,67 +353,14 @@ int fill_smap(smap_entry * sh_smap, int num)
 }
 
 
-struct inode_operations {
-  int (*create) (int *,int *,int);
-  int * (*lookup) (int *,int *);
-  int (*link) (int *,int *,int *);
-  int (*unlink) (int *,int *);
-  int (*symlink) (int *,int *,const char *);
-  int (*mkdir) (int *,int *,int);
-  int (*rmdir) (int *,int *);
-  int (*mknod) (int *,int *,int,int);
-  int (*rename) (int *, int *,
-                 int *, int *);
-  int (*readlink) (int *, char *,int);
-  int (*follow_link) (int *, int *);
-  void (*truncate) (int *);
-  int (*permission) (int *, int);
-  int (*revalidate) (int *);
-  /*
-    int (*setattr) (int *, int *);
-    int (*getattr) (int *, int *);
-    int (*setxattr) (int *, const char *, void *, size_t, int);
-    ssize_t (*getxattr) (int *, const char *, void *, size_t);
-    ssize_t (*listxattr) (int *, char *, size_t);
-    int (*removexattr) (int *, const char *);
-  */
-};
-
-struct file_operations {
-  int (*create) (int *,int *,int);
-};
-
-struct proc_dir_entry {
-  unsigned short low_ino;
-  unsigned short namelen;
-  const char * name;
-  mode_t mode;
-  nlink_t nlink;
-  uid_t uid;
-  gid_t gid;
-  unsigned long size; 
-  struct inode_operations * proc_iops;
-  struct file_operations * proc_fops;
-  /*
-  get_info_t *get_info;
-  struct module *owner;
-  struct proc_dir_entry *next, *parent, *subdir;
-  void *data;
-  read_proc_t *read_proc;
-  write_proc_t *write_proc;
-  atomic_t count;         
-  int deleted;  
-  */          
-};
-
-
-
 int main(int argc, char * argv[])
 {
   int i, count, maxcall, qq;
   int which = 4;
+  int two_six_seventeen_plus = 0;
   smap_entry sh_smap[SH_MAXCALLS];
   struct utsname utbuf;
+  char *p = NULL;
 
   unsigned long proc_root;
   unsigned long proc_root_iops;
@@ -377,17 +383,36 @@ int main(int argc, char * argv[])
       exit (EXIT_FAILURE);
     }
 
-  if      (strncmp(utbuf.release, "2.2", 3) == 0)
+  if (strncmp(utbuf.release, SH_KERNEL_VERSION, 3) != 0)
+    {
+      fprintf(stderr, "kern_head: current kernel version %s does not match\n",
+	      utbuf.release);
+      fprintf(stderr, "kern_head: %s from config.h\n", SH_KERNEL_VERSION);
+      fprintf(stderr, "kern_head: continuing with %s\n", SH_KERNEL_VERSION);
+
+      p = SH_KERNEL_VERSION;
+    } else {
+      p = utbuf.release;
+    }
+
+  if      (strncmp(p, "2.2", 3) == 0)
     which = 2;
-  else if (strncmp(utbuf.release, "2.4", 3) == 0)
+  else if (strncmp(p, "2.4", 3) == 0)
     which = 4;
-  else if (strncmp(utbuf.release, "2.6", 3) == 0)
-    which = 6;
+  else if (strncmp(p, "2.6", 3) == 0)
+    {
+      which = 6;
+      if (17 >= atoi (&p[4]))
+	{
+	  two_six_seventeen_plus = 1;
+	}
+    }
   else
     {
-      fprintf(stderr, "kern_head: kernel %s not supported\n", utbuf.release);
+      fprintf(stderr, "kern_head: kernel %s not supported\n", p);
       exit (EXIT_FAILURE);
     }
+
   
   if (utbuf.machine[0] != 'i' || utbuf.machine[2] != '8' || 
       utbuf.machine[3] != '6')
@@ -413,11 +438,11 @@ int main(int argc, char * argv[])
   printf("#define SH_KERN_CALLS_H\n\n");
 
   printf("\n/* Kernel %s, machine %s -- use table %s */\n\n", 
-	 utbuf.release, utbuf.machine,
+	 p, utbuf.machine,
 	 (which == 2) ? "callz_2p2" : "callz_2p4");
       
 
-  /* initialite the system call table 
+  /* initiate the system call table 
    */
   for (i = 0; i < SH_MAXCALLS; ++i)
     {
@@ -486,7 +511,7 @@ int main(int argc, char * argv[])
       if (sh_smap[i].addr == 0UL)
         {
           if (verbose > 0)
-            fprintf(stdout, "** unknown syscall **: [%s]\n", sh_smap[i].name);
+            fprintf(stderr, "** unknown syscall **: [%s]\n", sh_smap[i].name);
           strcpy(sh_smap[i].name, "sys_ni_syscall");
           sh_smap[i].addr = addr_ni_syscall;
         }
@@ -596,6 +621,10 @@ int main(int argc, char * argv[])
     printf("#define PROC_ROOT_IOPS_LOC %#lx\n\n", proc_root_iops);
   }
 
+  if (two_six_seventeen_plus == 1) {
+    printf("#define TWO_SIX_SEVENTEEN_PLUS 1\n\n");
+  }
+
   printf("#endif\n");
 
   exit (EXIT_SUCCESS);
@@ -693,10 +722,19 @@ int main(int argc, char * argv[])
       exit (EXIT_FAILURE);
     }
 
+#ifdef __OpenBSD__
+  if      (utbuf.release[0] == '3')
+    which = 38;
+  else if (utbuf.release[0] == '4')
+    which = 40;
+#else
   if      (utbuf.release[0] == '4')
     which = 4;
   else if (utbuf.release[0] == '5')
     which = 5;
+  else if (utbuf.release[0] == '6')
+    which = 5;
+#endif
   else
     {
       fprintf(stderr, "kern_head: kernel %s not supported\n", utbuf.release);
@@ -714,7 +752,7 @@ int main(int argc, char * argv[])
     {
       fprintf(stderr, "\n");
       fprintf(stderr, "NOTE:  kern_head: must run as 'root' ");
-      fprintf(stderr, "(need to read from /dev/kmem)\n");
+      fprintf(stderr, "(need to read from kernel)\n");
       fprintf(stderr, "       If you get this message, then proceed ");
       fprintf(stderr, "as follows:\n");
       fprintf(stderr, "       $ su\n");
@@ -737,13 +775,27 @@ int main(int argc, char * argv[])
       fprintf(stderr, "check_sysent: kvm_nlist: %s\n", kvm_geterr(kd));
       exit(EXIT_FAILURE);
     }
+  else if (i == 1)
+    {
+      fprintf(stderr, "check_sysent: kvm_nlist: _sysent not found\n");
+      exit(EXIT_FAILURE);
+    }
+  else if (list[0].n_value == 0)
+    {
+      fprintf(stderr, "check_sysent: kvm_nlist: zero address for _sysent\n");
+      exit(EXIT_FAILURE);
+    }
 
   if (which == 4)
     printf("\n/* Kernel %s, machine %s -- use table %s */\n\n", 
         	 utbuf.release, utbuf.machine, "callz_fbsd");
-  if (which == 5)
+  else if (which == 5 || which == 6)
     printf("\n/* Kernel %s, machine %s -- use table %s */\n\n",
                  utbuf.release, utbuf.machine, "callz_fbsd5");
+  else if (which == 38 || which == 40)
+    printf("\n/* Kernel %s, machine %s -- use table %s */\n\n",
+                 utbuf.release, utbuf.machine, "callz_obsd");
+      
       
   i = 0;
   if (which == 4) {
@@ -756,9 +808,9 @@ int main(int argc, char * argv[])
     if ((utbuf.release[1] == '.') && (utbuf.release[2] == '1') && 
 	(utbuf.release[3] == '0'))
       {
-	sys_list[336].n_name = callz_fbsd[290]; /* sendfile -> nosys */
+	sys_list[336].n_name = callz_fbsd[151]; /* sendfile -> nosys */
       }
-  } else {
+  } else if (which == 5 || which == 6) {
     while ((callz_fbsd5[i] != NULL) && (i < SYS_MAXSYSCALL))
       {
 	sys_list[i].n_name = callz_fbsd5[i];
@@ -766,6 +818,15 @@ int main(int argc, char * argv[])
 	++i;
       }
   }
+  else if (which == 38 || which == 40) {
+    while ((callz_obsd[i] != NULL) && (i < SYS_MAXSYSCALL))
+      {
+	sys_list[i].n_name = callz_obsd[i];
+	/* fprintf(stderr, "sys_list[%d] = %s\n", i, sys_list[i].n_name); */
+	++i;
+      }
+  }
+  
   count = i;
   sys_list[i].n_name = NULL;
    
@@ -779,7 +840,7 @@ int main(int argc, char * argv[])
      {
 	fprintf(stderr, "check_sysent: kvm_nlist: %d out of %d invalid.\n",
 		i, count);
-	fprintf(stderr, "              Probably callz_fbsd in kern_head.c\n");
+	fprintf(stderr, "              Probably the table in kern_head.h\n");
 	fprintf(stderr, "              is not for your kernel version.\n");
 	fprintf(stderr, "              (No reason to worry, kcheck will "\
                                        "work anyway)\n\n");
@@ -812,7 +873,7 @@ int main(int argc, char * argv[])
       if((unsigned long)sy.sy_call != sys_list[i].n_value && 
 	 sys_list[i].n_value != 0 &&
 	 0 != strcmp(sys_list[i].n_name, "_nosys") &&
-	 (unsigned long)sy.sy_call != sys_list[290].n_value)  
+	 (unsigned long)sy.sy_call != sys_list[151].n_value)  
 	{
           fprintf(stderr,
                   "WARNING: (kvm_nlist) %#lx != %#lx (sysent[%03d])  %03d [%s]\n",
