@@ -116,7 +116,7 @@ do_read( int fd, void *buf, int nbytes )
 
 int sh_entropy(int getbytes, char * nbuf)
 {
-    static int fd = -1;
+    int fd = -1;
     int n;
     byte buffer[256+2];
     int nbytes;
@@ -137,7 +137,7 @@ int sh_entropy(int getbytes, char * nbuf)
   restart:
     if( do_restart ) {
         if( fd != -1 ) {
-            close( fd );
+            sl_close_fd(FIL__, __LINE__,  fd );
             fd = -1;
         }
     }
@@ -146,10 +146,11 @@ int sh_entropy(int getbytes, char * nbuf)
         char *name;
         struct sockaddr_un addr;
         int addr_len;
+	int retval;
 
-      #ifdef EGD_SOCKET_NAME
+#ifdef EGD_SOCKET_NAME
         bname = EGD_SOCKET_NAME;
-      #endif
+#endif
         if ( !bname || !*bname )
             bname = _("=entropy");
 
@@ -169,7 +170,7 @@ int sh_entropy(int getbytes, char * nbuf)
 
         memset( &addr, 0, sizeof(addr) );
         addr.sun_family = AF_UNIX;
-        strcpy( addr.sun_path, name );              /* known to fit  */
+        sl_strlcpy( addr.sun_path, name, sizeof(addr.sun_path) );
         addr_len = offsetof( struct sockaddr_un, sun_path )
                    + strlen( addr.sun_path );
 
@@ -183,13 +184,17 @@ int sh_entropy(int getbytes, char * nbuf)
 	    SH_FREE(name);
 	    SL_RETURN( -1, _("sh_entropy") );
 	  }
-        if( connect( fd, (struct sockaddr*)&addr, addr_len) == -1 )
+	do {
+	  retval = connect(fd, (struct sockaddr *) &sinr, sizeof(sinr));
+	} while (retval < 0 && (errno == EINTR || errno == EINPROGRESS));
+        if( retval == -1 )
 	  {
 	    myerror = errno;
 	    sh_error_handle ((-1), FIL__, __LINE__, myerror, MSG_E_SUBGEN,
 			     _("cannot connect to unix domain socket"),
 			     _("sh_entropy") ); 
 	    SH_FREE(name);
+	    sl_close_fd(FIL__, __LINE__, fd);
 	    SL_RETURN( -1, _("sh_entropy") );
 	  }
         SH_FREE(name);
@@ -205,7 +210,8 @@ int sh_entropy(int getbytes, char * nbuf)
 	    myerror = errno;
 	    sh_error_handle ((-1), FIL__, __LINE__, myerror, MSG_E_SUBGEN,
 			     _("cannot write to EGD"),
-			     _("sh_entropy") ); 
+			     _("sh_entropy") );
+	    sl_close_fd(FIL__, __LINE__, fd);
 	    SL_RETURN( -1, _("sh_entropy") );
 	  }
     n = do_read( fd, buffer, 1 );
@@ -246,7 +252,8 @@ int sh_entropy(int getbytes, char * nbuf)
 	    myerror = errno;
 	    sh_error_handle ((-1), FIL__, __LINE__, myerror, MSG_E_SUBGEN,
 			     _("cannot write to EGD"),
-			     _("sh_entropy") ); 
+			     _("sh_entropy") );
+	    sl_close_fd(FIL__, __LINE__, fd);
 	    SL_RETURN( -1, _("sh_entropy") );
 	  }
         n = do_read( fd, buffer, nbytes );
@@ -266,7 +273,7 @@ int sh_entropy(int getbytes, char * nbuf)
         length -= n;
     }
     memset(buffer, 0, sizeof(buffer) );
-
+    sl_close_fd(FIL__, __LINE__, fd);
     SL_RETURN( 0, _("sh_entropy") ); /* success */
 }
 
@@ -275,95 +282,26 @@ int sh_entropy(int getbytes, char * nbuf)
 
 #if defined (HAVE_URANDOM)
 
-#include <setjmp.h>
+#include "sh_pthread.h"
 
-static jmp_buf          entropy_timeout;
-
-static void sh_entropy_alarmhandle (int mysignal)
+int read_mbytes(int timeout_val, const char * path, char * nbuf, int nbytes)
 {
-  (void) mysignal; /* avoid compiler warning */
-  longjmp (entropy_timeout, 1);
-}
-
-
-int read_mbytes(int timeout_val, char * path, char * nbuf, int nbytes)
-{
-  int count, m_count;
+  int m_count;
   int fd2;
 
-  struct  sigaction  new_act;
-  sigset_t           unblock;
-
-  struct sigaction      old_act;
-  volatile unsigned int old_alarm = 0;
-
-  new_act.sa_handler = sh_entropy_alarmhandle;
-  sigemptyset( &new_act.sa_mask );       /* set an empty mask       */
-  new_act.sa_flags = 0;                  /* init sa_flags           */
-
-  sigemptyset(&unblock);
-  sigaddset  (&unblock, SIGALRM);
-
   SL_ENTER(_("read_mbytes"));
 
   if ((fd2 = aud_open (FIL__, __LINE__, SL_NOPRIV, path, O_RDONLY, 0)) >= 0) 
     {
       /* Test whether file is a character device, and is 
-       * not world writeable.
+       * readable.
        */
-      if (0 == sh_unix_file_exists(fd2)) 
+      if (0 == sh_unix_device_readable(fd2)) 
 	{
-
-	  /* alarm was triggered
-	   */
-	  if (setjmp(entropy_timeout) != 0)
-	    {
-	      alarm(0);
-	      sigaction (SIGALRM, &old_act, NULL);
-	      alarm(old_alarm);
-	      sigprocmask(SIG_UNBLOCK, &unblock, NULL);
-	      TPT((0,FIL__,__LINE__, _("msg=<read_mbytes: timeout>\n"))); 
-	      close (fd2);
-	      SL_RETURN(0, _("read_mbytes"));
-	    }
-
-	  /* timeout after 30 seconds
-	   */
-	  old_alarm = alarm(0);
-	  sigaction (SIGALRM, &new_act, &old_act);
-	  alarm(timeout_val);
-
-	  m_count = 0;
-
-	  while (m_count < nbytes) 
-	    {
-	      errno = 0; /* paranoia */
-	      count = read (fd2, &nbuf[m_count], nbytes-m_count);
-
-	      switch (count)
-		{
-		case -1:
-#ifdef EWOULDBLOCK
-                  if (errno == EINTR || errno == EAGAIN ||
-                      errno == EWOULDBLOCK)
-#else
-                  if (errno == EINTR || errno == EAGAIN)
-#endif
-                    continue;
-
-                  /* if errno == -1 && no continue: fallthrough to this */
-                case 0:
-                  break;
-                default:
-                  m_count += count;
-                }
-	    }
-	  close (fd2);
-
-	  alarm(0);
-	  sigaction (SIGALRM, &old_act, NULL);
-	  alarm(old_alarm);
-	  sigprocmask(SIG_UNBLOCK, &unblock, NULL);
+	  m_count = sl_read_timeout_fd(fd2, nbuf, nbytes, 
+				       timeout_val, SL_FALSE);
+	  if (m_count < 0)
+	    m_count = 0;
 	}
       else
 	m_count = 0;
@@ -371,6 +309,8 @@ int read_mbytes(int timeout_val, char * path, char * nbuf, int nbytes)
   else
     m_count = 0;
 
+  sl_close_fd(FIL__, __LINE__, fd2);
+
   TPT((0, FIL__, __LINE__, _("msg=<read_mbytes: OK>\n"))); 
   SL_RETURN(m_count, _("read_mbytes"));
 }
@@ -383,6 +323,7 @@ int sh_entropy(int nbytes, char * nbuf)
 {
   int    i, m_count = 0;
   char * keybuf;
+  UINT32 kbuf[KEY_BYT/sizeof(UINT32)];
   char   addbuf[2 * KEY_BYT];
 
   SL_ENTER(_("sh_entropy"));
@@ -395,7 +336,7 @@ int sh_entropy(int nbytes, char * nbuf)
   memset(nbuf, '\0', nbytes);
 
 #ifdef NAME_OF_DEV_URANDOM
-  m_count = read_mbytes (30, NAME_OF_DEV_RANDOM, nbuf, nbytes);
+  m_count = read_mbytes (  1, NAME_OF_DEV_RANDOM, nbuf, nbytes);
 #else
   m_count = read_mbytes (300, NAME_OF_DEV_RANDOM, nbuf, nbytes);
 #endif
@@ -434,18 +375,22 @@ int sh_entropy(int nbytes, char * nbuf)
       for (i = 0; i < KEY_BYT; ++i)
 	addbuf[i+KEY_BYT] = skey->poolv[i];
       keybuf = (char *) sh_tiger_hash_uint32 (addbuf, 
-					      TIGER_DATA, 2 * KEY_BYT);
+					      TIGER_DATA, 2 * KEY_BYT,
+					      kbuf, KEY_BYT/sizeof(UINT32));
       memset(addbuf, '\0', sizeof(addbuf));
       
       /* -- Give out nbytes bytes from the new pool. --
        */
+      SH_MUTEX_LOCK_UNSAFE(mutex_skey);
       for (i = 0; i < KEY_BYT; ++i)
 	{
 	  skey->poolv[i] = keybuf[i];
 	  if (i < nbytes) 
 	    nbuf[i] = keybuf[i];
 	}
+      SH_MUTEX_UNLOCK_UNSAFE(mutex_skey);
       memset (keybuf, '\0', KEY_BYT);
+      memset (kbuf,   '\0', sizeof(kbuf));
       
       SL_RETURN(0, _("sh_entropy"));
     }
@@ -474,9 +419,11 @@ int sh_entropy(int nbytes, char * nbuf)
 #endif
 
 #include "sh_static.h"
+#include "sh_pthread.h"
 
 static
 char   * com_path[] = {
+  N_("/usr/bin/xpg4/"),
   N_("/usr/ucb/"),
   N_("/bin/"),
   N_("/sbin/"),
@@ -497,7 +444,7 @@ typedef struct {
 } sourcetable_t;
 
 static
-sourcetable_t source[] = {
+sourcetable_t source_template[] = {
   { N_("w"),
     N_("w"),
     0,
@@ -511,7 +458,7 @@ sourcetable_t source[] = {
     0,
     NULL },
   { N_("ps"),
-    N_("ps aux"),
+    N_("ps -ef"),
     0,
     0,
     0,
@@ -522,12 +469,6 @@ sourcetable_t source[] = {
     0,
     0,
     NULL },
-  { N_("df"),
-    N_("df"),
-    0,
-    0,
-    0,
-    NULL },
   { N_("free"),
     N_("free"),
     0,
@@ -572,22 +513,27 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
   int i;
   int pipedes[2];
   FILE *outf = NULL;
-  struct passwd * tempres;
   char * arg[4];
   char * envp[2];
+  size_t len;
+  char   arg0[80];
+  char   arg1[80];
 
   SL_ENTER(_("sh_popen"));
 
-  arg[0] = _("/bin/sh");
-  arg[1] = _("-c");
+  strncpy (arg0, _("/bin/sh"), sizeof(arg0));
+  arg[0] = arg0;
+  strncpy (arg1, _("-c"), sizeof(arg1));
+  arg[1] = arg1;
   arg[2] = command;
   arg[3] = NULL;
 
   if (sh.timezone != NULL)
     {
-      envp[0] = malloc (sl_strlen(sh.timezone) + 4);     /* free() ok     */
+      len = sl_strlen(sh.timezone) + 4;
+      envp[0] = malloc (len);     /* free() ok     */
       if (envp[0] != NULL)
-	sprintf (envp[0], "TZ=%s", sh.timezone);         /* known to fit  */
+	sl_snprintf (envp[0], len, "TZ=%s", sh.timezone);
       else
 	envp[0] = NULL;
       envp[1] = NULL;
@@ -605,34 +551,40 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
     SL_RETURN(NULL, _("sh_popen"));
   }
   
+  fflush (NULL);
+
   source->pid = aud_fork(FIL__, __LINE__);
   
   /* Failure
    */
   if (source->pid == (pid_t) - 1) {
-    close(pipedes[0]);
-    close(pipedes[1]);
+    sl_close_fd(FIL__, __LINE__, pipedes[0]);
+    sl_close_fd(FIL__, __LINE__, pipedes[1]);
     if (envp[0] != NULL) free(envp[0]);
     SL_RETURN(NULL, _("sh_popen"));
   }
 
   if (source->pid == (pid_t) 0) 
     {
+      int val_return;
 
       /* child - make read side of the pipe stdout 
        */
-      if (retry_aud_dup2(FIL__, __LINE__, 
-			 pipedes[STDOUT_FILENO], STDOUT_FILENO) < 0)
-	aud__exit(FIL__, __LINE__, EXIT_FAILURE);
+      do {
+	val_return = dup2 (pipedes[STDOUT_FILENO], STDOUT_FILENO);
+      } while (val_return < 0 && errno == EINTR);
+
+      if (val_return < 0)
+	_exit(EXIT_FAILURE);
       
       /* close the pipe descriptors 
        */
-      close   (pipedes[STDIN_FILENO]);
-      close   (pipedes[STDOUT_FILENO]);
+      sl_close_fd   (FIL__, __LINE__, pipedes[STDIN_FILENO]);
+      sl_close_fd   (FIL__, __LINE__, pipedes[STDOUT_FILENO]);
 
       /* don't leak file descriptors
        */
-      sh_unix_closeall (3, -1);
+      sh_unix_closeall (3, -1, SL_TRUE); /* in child process */
 
       /* zero priv info
        */
@@ -641,37 +593,54 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
       /* drop root privileges
        */
       i = 0; 
-      if (0 == geteuid()) {  
-	tempres = sh_getpwnam(DEFAULT_IDENT);
-	if (NULL != tempres) {
-	  i = aud_setgid(FIL__, __LINE__, tempres->pw_gid); 
-	  if (i == 0)
-	    i = sh_unix_initgroups(DEFAULT_IDENT ,tempres->pw_gid);
-	  if (i == 0) 
-	    i = aud_setuid(FIL__, __LINE__, tempres->pw_uid);
-	  /* make sure we cannot get root again
-	   */
-	  if ((tempres->pw_uid != 0) && (aud_setuid(FIL__, __LINE__, 0) >= 0))
+      if (0 == geteuid()) 
+	{
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+	  struct passwd    pwd;
+	  char           * buffer = SH_ALLOC(SH_PWBUF_SIZE);
+	  struct passwd *  tempres;
+	  sh_getpwnam_r(DEFAULT_IDENT, &pwd, buffer, SH_PWBUF_SIZE, &tempres);
+#else
+	  struct passwd * tempres = sh_getpwnam(DEFAULT_IDENT);
+#endif
+  
+	  if (NULL != tempres) {
+	    i = setgid(tempres->pw_gid); 
+	    if (i == 0)
+	      i = sh_unix_initgroups(DEFAULT_IDENT ,tempres->pw_gid);
+	    if (i == 0) 
+	      i = setuid(tempres->pw_uid);
+	    /* make sure we cannot get root again
+	     */
+	    if ((tempres->pw_uid != 0) && 
+		(setuid(0) >= 0))
+	      i = -1;
+	  } else {
 	    i = -1;
-	} else {
-	  i = -1;
+	  }
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+	  SH_FREE(buffer);
+#endif
 	}
-      }
       
       /* some problem ...
        */
       if (i == -1) {
-	aud__exit(FIL__, __LINE__, EXIT_FAILURE);
+	_exit(EXIT_FAILURE);
       }
       
-      freopen (_("/dev/null"), "r+", stderr);
-      
-      /* exec the program */
-      retry_aud_execve (FIL__, __LINE__, _("/bin/sh"), arg, envp);
+      if (NULL != freopen (_("/dev/null"), "r+", stderr))
+	{
       
+	  /* exec the program */
+	  do {
+	    val_return = execve (_("/bin/sh"), arg, envp);
+	  } while (val_return < 0 && errno == EINTR);
+	}
+
       /* failed 
        */
-      aud__exit(FIL__, __LINE__, EXIT_FAILURE);
+      _exit(EXIT_FAILURE);
     }
 
     /* parent
@@ -679,7 +648,7 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
     if (envp[0] != NULL) 
       free(envp[0]);
 
-    close (pipedes[STDOUT_FILENO]);
+    sl_close_fd (FIL__, __LINE__, pipedes[STDOUT_FILENO]);
     retry_fcntl (FIL__, __LINE__, pipedes[STDIN_FILENO], F_SETFD, FD_CLOEXEC);
 
     outf = fdopen (pipedes[STDIN_FILENO], "r");
@@ -687,7 +656,7 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
     if (outf == NULL) 
       {
         aud_kill (FIL__, __LINE__, source->pid, SIGKILL);
-	close (pipedes[STDOUT_FILENO]);
+	sl_close_fd (FIL__, __LINE__, pipedes[STDOUT_FILENO]);
         waitpid (source->pid, NULL, 0);
         source->pid = 0;
 	SL_RETURN(NULL, _("sh_popen"));
@@ -700,17 +669,44 @@ static FILE * sh_popen (sourcetable_t  *source, char * command)
 static int sh_pclose (sourcetable_t *source)
 {
     int status = 0;
+    int retval;
+    char msg[128];
+    char errbuf[SH_ERRBUF_SIZE];
 
     SL_ENTER(_("sh_pclose"));
 
-    status = fclose(source->pipe);
-    if (status)
+    retval = sl_fclose(FIL__, __LINE__, source->pipe);
+    if (retval)
       {
+	sh_error_handle (SH_ERR_ALL, FIL__, __LINE__, retval, 
+			 MSG_E_SUBGEN,
+			 sh_error_message(retval, errbuf, sizeof(errbuf)),
+                         _("sh_pclose"));
 	SL_RETURN((-1), _("sh_pclose"));
       }
 
-    if (waitpid(source->pid, NULL, 0) != source->pid)
-      status = -1;
+    retval = waitpid(source->pid, &status, 0);
+    if (retval != source->pid)
+      {
+	sh_error_handle (SH_ERR_ALL, FIL__, __LINE__, retval, 
+			 MSG_E_SUBGEN,
+			 sh_error_message(retval, errbuf, sizeof(errbuf)),
+                         _("sh_pclose"));
+
+	status = -1;
+      }
+#if !defined(USE_UNO)
+    else if (WIFSIGNALED(status))
+      {
+	sl_snprintf(msg, sizeof(msg), _("Subprocess terminated by signal %d"),
+		    WTERMSIG(status));
+	sh_error_handle (SH_ERR_ALL, FIL__, __LINE__, retval, 
+			 MSG_E_SUBGEN,
+			 msg,
+                         _("sh_pclose"));
+	status = -1;
+      }
+#endif
 
     source->pipe = NULL;
     source->pid = 0;
@@ -733,6 +729,7 @@ int sh_entropy(int nbytes, char * nbuf)
   int    count;
 
   char * keybuf;
+  UINT32 kbuf[KEY_BYT/sizeof(UINT32)];
   char   addbuf[2 * KEY_BYT];
 
   struct timeval tv;
@@ -740,7 +737,10 @@ int sh_entropy(int nbytes, char * nbuf)
   unsigned long select_now = 0;
   int    maxFD = 0;
   int    imax, selcount;
+  char errbuf[SH_ERRBUF_SIZE];
 
+  sourcetable_t  *source = NULL;
+  
   SL_ENTER(_("sh_entropy"));
 
   ASSERT((nbytes <= KEY_BYT), _("nbytes <= KEY_BYT"))
@@ -751,6 +751,7 @@ int sh_entropy(int nbytes, char * nbuf)
 
   /* --- If there is entropy in the pool, return it. ---
    */
+  SH_MUTEX_LOCK_UNSAFE(mutex_skey);
   if (skey->poolc >= nbytes)
     {
       j = KEY_BYT - skey->poolc;
@@ -759,8 +760,10 @@ int sh_entropy(int nbytes, char * nbuf)
 	  nbuf[i] = skey->poolv[i+j];
 	  --skey->poolc;
 	}
+      SH_MUTEX_UNLOCK_UNSAFE(mutex_skey); /* alternative path */
       SL_RETURN(0, _("sh_entropy"));
     }
+  SH_MUTEX_UNLOCK_UNSAFE(mutex_skey);
 
 
   FD_ZERO(&fds);   
@@ -771,11 +774,19 @@ int sh_entropy(int nbytes, char * nbuf)
   if (0 != (caperr = sl_get_cap_sub()))
     {
       sh_error_handle((-1), FIL__, __LINE__, caperr, MSG_E_SUBGEN,
-		      sh_error_message (caperr), 
+		      sh_error_message (caperr, errbuf, sizeof(errbuf)), 
 		      _("sl_get_cap_sub"));
     }
 
-  while (source[i].command != NULL) {
+  while (source_template[i].command != NULL) {
+    ++i;
+  }
+  source = SH_ALLOC(i * sizeof(sourcetable_t));
+  for (j = 0; j < i;++j)
+    memcpy(&source[j], &source_template[j], sizeof(sourcetable_t));
+  i = 0;
+
+  while (source_template[i].command != NULL) {
 
     j = 0;
     while (com_path[j] != NULL)
@@ -783,6 +794,7 @@ int sh_entropy(int nbytes, char * nbuf)
 	sl_strlcpy(combuf, _(com_path[j]),       80);
 	sl_strlcat(combuf, _(source[i].command), 80);
 
+	/* flawfinder: ignore */
 	if ( access (combuf, X_OK) == 0) 
 	  {
 	    sl_strlcpy(combuf, _(com_path[j]),       80);
@@ -848,7 +860,7 @@ int sh_entropy(int nbytes, char * nbuf)
 	++select_now;
 	sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_ENTOUT,
 			 (unsigned long) select_now);
-	if ( select_now > 3 ) 
+	if ( select_now > 9 ) 
 	  break;
       }
     
@@ -904,18 +916,21 @@ int sh_entropy(int nbytes, char * nbuf)
 	}
     }
   buffer[bufcount] = '\0';
+  
+  SH_FREE(source);
 
   if (0 != (caperr = sl_drop_cap_sub()))
     {
       sh_error_handle((-1), FIL__, __LINE__, caperr, MSG_E_SUBGEN,
-		      sh_error_message (caperr), 
+		      sh_error_message (caperr, errbuf, sizeof(errbuf)), 
 		      _("sl_drop_cap_sub"));
     }
 
   if (bufcount > 0) 
     {
       keybuf = (char *) sh_tiger_hash_uint32 (buffer, 
-					      TIGER_DATA, sl_strlen(buffer));
+					      TIGER_DATA, sl_strlen(buffer),
+					      kbuf, KEY_BYT/sizeof(UINT32));
 
       /* add previous entropy into the new pool
        */
@@ -926,14 +941,17 @@ int sh_entropy(int nbytes, char * nbuf)
 	  addbuf[i+KEY_BYT] = skey->poolv[i];
 	}
       keybuf = (char *) sh_tiger_hash_uint32 (addbuf, 
-					      TIGER_DATA, sizeof(addbuf));
+					      TIGER_DATA, sizeof(addbuf),
+					      kbuf, KEY_BYT/sizeof(UINT32));
       memset(addbuf, '\0', sizeof(addbuf));
       
       /* store in system pool
        */
+      SH_MUTEX_LOCK_UNSAFE(mutex_skey);
       for (i = 0; i < KEY_BYT; ++i)
 	skey->poolv[i] = keybuf[i];
       skey->poolc = KEY_BYT;
+      SH_MUTEX_UNLOCK_UNSAFE(mutex_skey);
       memset (buffer, '\0', BUF_ENT+2);
       memset (keybuf, '\0', KEY_BYT);
       SH_FREE(buffer);
@@ -946,11 +964,13 @@ int sh_entropy(int nbytes, char * nbuf)
 
   /* give out nbytes Bytes from the entropy pool
    */
+  SH_MUTEX_LOCK_UNSAFE(mutex_skey);
   for (i = 0; i < nbytes; ++i)
     {
       nbuf[i] = skey->poolv[i];
       --skey->poolc;
     }
+  SH_MUTEX_UNLOCK_UNSAFE(mutex_skey);
 
   SL_RETURN(0, _("sh_entropy"));
 }
@@ -958,6 +978,30 @@ int sh_entropy(int nbytes, char * nbuf)
 /* HAVE_UNIX_RANDOM */
 #endif
 
+#ifdef SH_CUTEST
+#include "CuTest.h"
+
+void Test_entropy (CuTest *tc)
+{
+  char                 bufx[9 * sizeof(UINT32) + 1];
+  char                 bufy[9 * sizeof(UINT32) + 1];
+  int                  status;
+
+  memset(skey->poolv, '\0', KEY_BYT);
+
+  status = sh_entropy (24, bufx);
+  CuAssertTrue(tc, 0 == status);
+
+  memset(skey->poolv, '\0', KEY_BYT);
+
+  status = sh_entropy (24, bufy);
+  CuAssertTrue(tc, 0 == status);
+
+  CuAssertTrue(tc, 0 != memcmp(bufx, bufy, 24));
+}
+#endif
+
+
 
 
 
