@@ -68,7 +68,7 @@ static int get_key_from_file(char * path, char * keyid, char * key)
 
   /* open the file, then check it 
    */
-  if ( SL_ISERROR(fd = sl_open_read (path, SL_NOPRIV)))
+  if ( SL_ISERROR(fd = sl_open_read (FIL__, __LINE__, path, SL_NOPRIV)))
     {
       fprintf(stderr, _("Could not open file <%s>\n"), path);
       _exit (EXIT_FAILURE);
@@ -115,7 +115,7 @@ static int get_key_from_file(char * path, char * keyid, char * key)
 
 static int just_list = S_FALSE;
 
-int sh_error_logverify_mod (char * s)
+int sh_error_logverify_mod (const char * s)
 {
   just_list = S_TRUE;
   if (s)      /* compiler warning (unused var) fix */
@@ -124,7 +124,7 @@ int sh_error_logverify_mod (char * s)
     return 0;
 } 
 
-int sh_error_logverify (char * s)
+int sh_error_logverify (const char * s)
 {
   SL_TICKET fd;
   int len;
@@ -144,6 +144,7 @@ int sh_error_logverify (char * s)
   char timestamp[64];
   char c_cont;
   int  chk_mode = CHK_KEY;
+  char hashbuf[KEYBUF_SIZE];
 
   sh_error_logoff();
 
@@ -161,10 +162,10 @@ int sh_error_logverify (char * s)
       _exit (EXIT_FAILURE);
     }
 
-  if ( SL_ISERROR(fd = sl_open_read (s, SL_NOPRIV)) )
+  if ( SL_ISERROR(fd = sl_open_read (FIL__, __LINE__, s, SL_NOPRIV)) )
     {
       fprintf(stderr, 
-	      _("FAIL: msg=\"File not accessible\", path=\"%s\"\n"), s);
+	      _("FAIL: msg=\"File not accessible\", error=\"%ld\", path=\"%s\"\n"), fd, s);
       _exit (EXIT_FAILURE);
     }
 
@@ -182,14 +183,14 @@ int sh_error_logverify (char * s)
 #endif
 #endif
 
-  buf  = (char *) SH_ALLOC( 2*SH_BUFSIZE+1 );
-  bufc = (char *) SH_ALLOC( 2*SH_BUFSIZE+1 );
+  buf  = (char *) SH_ALLOC( 2*SH_MSG_BUF+1 );
+  bufc = (char *) SH_ALLOC( 2*SH_MSG_BUF+1 );
 
   while (1 == 1) 
     {
       /* get the log message
        */
-      if (sh_unix_getline (fd, buf, (2*SH_BUFSIZE)) < 0) 
+      if (sh_unix_getline (fd, buf, (2*SH_MSG_BUF)) < 0) 
 	break;
 
       len = (int) sl_strlen(buf);
@@ -220,7 +221,7 @@ int sh_error_logverify (char * s)
 	   */
 	  start = 1;
 	  do {
-	    if ( sh_unix_getline (fd, buf, (2*SH_BUFSIZE)) < 0)
+	    if ( sh_unix_getline (fd, buf, (2*SH_MSG_BUF)) < 0)
 	      break;
 	  } while (buf[0] == '\0' || buf[0] == '\n');
 	  len = (int) sl_strlen(buf);
@@ -291,7 +292,7 @@ int sh_error_logverify (char * s)
       while (1 == 1)
 	{
 	  do {
-	    if ( sh_unix_getline (fd, bufc, (2*SH_BUFSIZE)) < 0)
+	    if ( sh_unix_getline (fd, bufc, (2*SH_MSG_BUF)) < 0)
 	      break;
 	  } while (bufc[0] == '\0' || bufc[0] == '\n');
 	  ++count;
@@ -299,9 +300,9 @@ int sh_error_logverify (char * s)
 	    {
 	      /* A continuation line. Add the newline. 
 	       */
-	      (void) sl_strlcat(buf, "\n", 2*SH_BUFSIZE+1);
+	      (void) sl_strlcat(buf, "\n", 2*SH_MSG_BUF+1);
 	      ++len;
-	      (void) sl_strlcat(buf, bufc, 2*SH_BUFSIZE+1);
+	      (void) sl_strlcat(buf, bufc, 2*SH_MSG_BUF+1);
 	      len += (int) sl_strlen(bufc);
 	    }
 	  else
@@ -378,28 +379,31 @@ int sh_error_logverify (char * s)
 	      while (sl_strlen(key) < KEY_LEN ) 
 		{ 
 		  if (key[0] != '\n' && key[0] != '\0')
-		    fprintf(stdout, _("New audit trail, enter key: "));
+		    fprintf(stdout, "%s",_("New audit trail, enter key: "));
 		  else if (key[0] == '\n')
 		    {
 		      (void) sl_strlcpy(key, 
-					sh_tiger_hash(NULL, TIGER_DATA, 0), 
+					sh_tiger_hash(NULL, TIGER_DATA, 0, 
+						      hashbuf, sizeof(hashbuf)), 
 					KEY_LEN+1);
 		      chk_mode = CHK_NON;
 		      break;
 		    }
 		  (void) fflush(stdout); 
 		  key[0] = '\0';
-		  (void) fgets(key, KEY_LEN+2, stdin);
-		  if (key[0] != '\n') 
+		  if (NULL != fgets(key, sizeof(key), stdin))
 		    {
-		      if (key[strlen(key) - 1] == '\n')
-			key[strlen(key) - 1] = '\0';
-		    }
-		  if (key[0] == '/')
-		    {
-		      chk_mode = CHK_FIL;
-		      (void) sl_strlcpy(path, key, KEY_LEN+1); 
-		      break;
+		      if (key[0] != '\n') 
+			{
+			  if (key[strlen(key) - 1] == '\n')
+			    key[strlen(key) - 1] = '\0';
+			}
+		      if (key[0] == '/')
+			{
+			  chk_mode = CHK_FIL;
+			  (void) sl_strlcpy(path, key, KEY_LEN+1); 
+			  break;
+			}
 		    }
 		}
 	    }
@@ -413,7 +417,7 @@ int sh_error_logverify (char * s)
 	      if (-1 == get_key_from_file(path, timestamp, key))
 		{
 		  chk_mode = CHK_KEY;
-		  fprintf(stdout, _("Key not found in file\n"));
+		  fprintf(stdout, "%s",_("Key not found in file\n"));
 		  goto findKey;
 		}
 	    }
@@ -427,11 +431,12 @@ int sh_error_logverify (char * s)
 	  /* Iterate the key.
 	   */
 	  (void) sl_strlcpy (key, 
-			     sh_tiger_hash (key, TIGER_DATA, KEY_LEN), 
+			     sh_tiger_hash (key, TIGER_DATA, KEY_LEN,
+					    hashbuf, sizeof(hashbuf)), 
 			     KEY_LEN+1);
 	}
       
-      (void) sl_strlcat ( buf, key, 2*SH_BUFSIZE + 1);
+      (void) sl_strlcat ( buf, key, 2*SH_MSG_BUF + 1);
       
 #ifdef SH_STEALTH
       sh_do_decode (signature, sl_strlen(signature));
@@ -439,7 +444,8 @@ int sh_error_logverify (char * s)
       
       status = sl_strncmp (signature, 
 			   sh_tiger_hash (buf, TIGER_DATA, 
-					  (unsigned long) sl_strlen(buf)),
+					  (unsigned long) sl_strlen(buf),
+					  hashbuf, sizeof(hashbuf)),
 			   KEY_LEN);
       
       buf[len] = '\0';    /* do not print out the key */
@@ -533,13 +539,13 @@ int sh_log_open (char * inet_peer,
 
   if (status == 0)
     {
-      fd = sl_open_write (logfile, SL_YESPRIV);
+      fd = sl_open_write (FIL__, __LINE__, logfile, SL_YESPRIV);
       if (SL_ISERROR(fd))
         {
 	  tmp  = sh_util_safe_name (logfile);
 	  (void) sl_get_euid(&uid);
           if ((*service_failure) == 0)
-            sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_ACCESS,
+            sh_error_handle ((-1), FIL__, __LINE__, fd, MSG_E_ACCESS,
                              (long) uid, tmp);
           status = -1;
         }
@@ -552,7 +558,9 @@ int sh_log_open (char * inet_peer,
       if (status < 0)
         {
 	  tmp  = sh_util_safe_name (logfile);
-	  len      = 6 + sl_strlen(tmp);
+	  len      = sl_strlen(tmp);
+	  if (sl_ok_adds (6, len))
+	    len += 6;
 	  lockfile = SH_ALLOC(len);
 	  (void) sl_strlcpy(lockfile,        tmp, len);
 	  (void) sl_strlcat(lockfile, _(".lock"), len);
@@ -568,8 +576,8 @@ int sh_log_open (char * inet_peer,
 
   if (status == 0)
     {
-      status = (long) SL_ISERROR(sl_forward(fd) ); 
-      if (status < 0)
+      status = sl_forward(fd); 
+      if (SL_ISERROR(status))
         {
 	  tmp  = sh_util_safe_name (logfile);
           (void) sl_get_euid(&uid);
@@ -603,7 +611,7 @@ typedef struct lfstc {
   int             log_start;
   char            sigkey_old[KEY_LEN+1];
   char            sigkey_new[KEY_LEN+1];
-  char            crypt[KEY_LEN+1]; 
+  char            crypto[KEY_LEN+1];
   struct  lfstc * next;
 } open_logfile;
 
@@ -612,7 +620,7 @@ static open_logfile * logfile_list = NULL;
 static int flag_sep_log = S_FALSE;
 
 #ifdef SH_WITH_SERVER
-int set_flag_sep_log (char * str)
+int set_flag_sep_log (const char * str)
 {
   return sh_util_flagval(str, &flag_sep_log);
 }
@@ -633,7 +641,7 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
   int                  store8;
 
   SL_TICKET            fd = -1;
-  long int             status;
+  size_t               status;
   struct _sh_log_buf   log_msg;
 
   char                 logfile[SH_PATHBUF+SH_MINIBUF+2];
@@ -641,7 +649,8 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
   open_logfile       * next    = NULL;
   char               * sigkey_new;
   char               * sigkey_old;
-  char               * crypt;
+  char               * crypto;
+  char                 hashbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("sh_log_file"));
 
@@ -655,7 +664,7 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
         {
 	  /* don't write second EOF mark
 	   */
-	  if (current->log_start != S_TRUE)
+	  if (current->log_start != S_TRUE && sh.flag.islocked == GOOD)
 	    {
 	      /* Don't use inet_peer == NULL, userwise a lock file will
 	       * be created.
@@ -733,7 +742,7 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
       current->log_start       = S_TRUE;
       memset(current->sigkey_old, (int)'\0', KEY_LEN+1);
       memset(current->sigkey_new, (int)'\0', KEY_LEN+1);
-      memset(current->crypt,      (int)'\0', KEY_LEN+1);
+      memset(current->crypto,     (int)'\0', KEY_LEN+1);
       current->next            = logfile_list;
       logfile_list             = current;
     }
@@ -748,13 +757,19 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
   /* --- Allocate storage and mlock it. ---
    */
 
-  status      = (long) sl_strlen (errmsg);
+  status      =  sl_strlen (errmsg);
+  if (!sl_ok_adds(status, (2*KEY_LEN)) || !sl_ok_adds((2*KEY_LEN + status),32))
+    {
+      sl_close(fd);
+      SL_RETURN ((-1), _("sh_log_file"));
+    }
+      
   log_msg.msg = (char *) SH_ALLOC ((size_t) (2*KEY_LEN + status + 32)); 
 
 #if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
   if (skey->mlock_failed == SL_FALSE) 
     {
-      if ( (-1) == sh_unix_mlock( log_msg.msg, 
+      if ( (-1) == sh_unix_mlock( FIL__, __LINE__, log_msg.msg, 
 				  (size_t)(2*KEY_LEN + status + 32) ) ) 
 	{
 	  skey->mlock_failed = SL_TRUE;
@@ -840,13 +855,13 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
     {
       sigkey_old = current->sigkey_old;
       sigkey_new = current->sigkey_new;
-      crypt      = current->crypt;
+      crypto     = current->crypto;
     }
   else
     {
       sigkey_old = skey->sigkey_old;
       sigkey_new = skey->sigkey_new;
-      crypt      = skey->crypt;
+      crypto     = skey->crypt;      /* flawfinder: ignore */
     }
 
   /* write the signature 
@@ -863,21 +878,22 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
       /* Hash the key to make sure it has the correct format.
        */
       (void) sl_strlcpy(sigkey_new, 
-			sh_tiger_hash (sigkey_old, TIGER_DATA, KEY_LEN), 
+			sh_tiger_hash (sigkey_old, TIGER_DATA, KEY_LEN,
+				       hashbuf, sizeof(hashbuf)), 
 			KEY_LEN+1);
 
       /* Copy it to 'crypt' for encryption.
        */
-      (void) sl_strlcpy(crypt, sigkey_new, KEY_LEN+1);
+      (void) sl_strlcpy(crypto, sigkey_new, KEY_LEN+1);
 
       /* Use message and compiled-in key to encrypt.
        */
       BREAKEXIT(sh_util_encode);
-      sh_util_encode(crypt, log_msg.msg, 0, 'B');
+      sh_util_encode(crypto, log_msg.msg, 0, 'B');
 
       /* Send out the key.
        */
-      (void) sl_strlcpy(log_msg.timestamp, sh_unix_time(0), KEY_LEN+1); 
+      (void) sh_unix_time(0, log_msg.timestamp, KEY_LEN+1); 
 
       store1               = errFlags.loglevel;
       store2               = errFlags.sysloglevel;
@@ -899,8 +915,8 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
       errFlags.preludelevel   = SH_ERR_NOT;
 
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_KEY_MAIL,
-		       sh.prg_name, crypt, 
-		       crypt, log_msg.timestamp);
+		       sh.prg_name, crypto, 
+		       crypto, log_msg.timestamp);
 
       /* send to other allowed channels
        */
@@ -912,7 +928,7 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
       errFlags.preludelevel   = store8;
 
       sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_START_KEY,
-		       sh.prg_name, crypt);
+		       sh.prg_name, crypto);
 
       /* Cleanup.
        */
@@ -925,7 +941,7 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
       errFlags.databaselevel  = store7;
 
 
-      memset (crypt, (int) '\0', KEY_LEN);
+      memset (crypto, (int) '\0', KEY_LEN);
       sh.flag.log_start    = S_FALSE;  
       current->log_start   = S_FALSE;
     } 
@@ -933,40 +949,44 @@ int  sh_log_file (/*@null@*/char *errmsg, /*@null@*/char * inet_peer)
     {
       log_msg.timestamp[0] = '\0';
       (void) sl_strlcpy (sigkey_new, 
-			 sh_tiger_hash (sigkey_old, TIGER_DATA, KEY_LEN),
+			 sh_tiger_hash (sigkey_old, TIGER_DATA, KEY_LEN, 
+					hashbuf, sizeof(hashbuf)),
 			 KEY_LEN+1);
     }
 
   /* --- Sign the message with the signature key. ---
    */
+  sh_tiger_hash (log_msg.msg, TIGER_DATA,
+		 (unsigned long)(status + KEY_LEN), 
+		 (char *) hashbuf, (size_t) sizeof(hashbuf));
 
   (void) sl_strlcat (log_msg.msg, sigkey_new, (size_t)(status + KEY_LEN + 2));
-  
   (void) sl_strlcpy (log_msg.signature,
-		     sh_tiger_hash (log_msg.msg, TIGER_DATA, 
-				    (unsigned long)(status + KEY_LEN)), 
+		     sh_tiger_hash (log_msg.msg, (TigerType) TIGER_DATA,
+				    (unsigned long)(status + KEY_LEN), 
+				    hashbuf, sizeof(hashbuf)),
 		     KEY_LEN+1);
   (void) sl_strlcpy (sigkey_old, sigkey_new, KEY_LEN+1); 
 
-  /*@-bufferoverflowhigh -usedef@*/
+  /*@-usedef@*/
 #ifdef SH_USE_XML
   if (log_msg.timestamp[0] != '\0')
-    sprintf(log_msg.sig,                            /* known to fit  */
+    sl_snprintf(log_msg.sig, sizeof(log_msg.sig),
 #ifdef FIX_XML
-	    _("\n<sig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
+		_("\n<sig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
 #else
-	    _("\nsig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
+		_("\nsig>%s%s</sig></log>\n"),          /* <sig> FIX XML */
 #endif
-	    log_msg.signature, log_msg.timestamp);
+		log_msg.signature, log_msg.timestamp);
   else
-    sprintf(log_msg.sig,                            /* known to fit  */
+    sl_snprintf(log_msg.sig, sizeof(log_msg.sig),
 #ifdef FIX_XML
-	    _("\n<sig>%s</sig></log>\n"),            /* <sig> FIX XML */
+		_("\n<sig>%s</sig></log>\n"),            /* <sig> FIX XML */
 #else
-	    _("\nsig>%s</sig></log>\n"),            /* <sig> FIX XML */
+		_("\nsig>%s</sig></log>\n"),            /* <sig> FIX XML */
 #endif
-	    log_msg.signature);
-  /*@+bufferoverflowhigh +usedef@*/
+		log_msg.signature);
+  /*@+usedef@*/
 
 #ifdef SH_STEALTH
   /* don't encode the line breaks (0 + last char)
