@@ -26,10 +26,14 @@
  * 23/04/2005 : R.W.: 
  *       include libprelude 0.9 code from Yoann Vandoorselaere
  */
- 
-#include "config_xor.h"
 
-#define _XOPEN_SOURCE 500 /* glibc2 needs this */
+
+/*
+ * for strptime()
+ */
+#define _GNU_SOURCE 1 
+
+#include "config_xor.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -53,9 +57,11 @@
 #include <unistd.h>
 #include <syslog.h>
 #include <pwd.h>
-#include <grp.h>
 
-#if defined(HAVE_LIBPRELUDE) && defined(HAVE_LIBPRELUDE_9)
+int     sh_argc_store;
+char ** sh_argv_store;
+
+#if defined(HAVE_LIBPRELUDE)
 
 
 /*
@@ -72,11 +78,13 @@
 /* 
  * includes for samhain-specific functions (sl_strstr, sh_error_handle)
  */
-#include "slib.h"
+#include "samhain.h"
 #include "sh_cat.h"
 #include "sh_error_min.h"
 #include "sh_prelude.h"
-
+#define SH_NEED_PWD_GRP 1
+#include "sh_static.h"
+char * sh_util_strdup (const char * str) SH_GNUC_MALLOC;
 /*
  * When SH_USE_XML is set, value are formated using name="value".
  * Otherwise, value is formatted using the format name=<value>.
@@ -92,9 +100,7 @@
 #define IDMEF_ANALYZER_MODEL _("Samhain")
 #define IDMEF_ANALYZER_CLASS _("Integrity Checker")
 #define IDMEF_ANALYZER_VERSION VERSION
-#define IDMEF_ANALYZER_MANUFACTURER _("Samhain by Rainer Wichmann")
-
-#define CLASSIFICATION_URL _("http://www.la-samhna.de/samhain/")
+#define IDMEF_ANALYZER_MANUFACTURER _("http://www.la-samhna.de/samhain/")
 
 
 
@@ -129,10 +135,22 @@ static void clear_and_set (int setpos, int flag)
         return;
 }
 
-static int set_prelude_severity_int (char * str, int prelude_sev)
+static int set_prelude_severity_int (const char * str, int prelude_sev)
 {
-        char * p = strtok (str, ", \t");
+        char * p;
+	char * dup = strdup (str);
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_STRTOK_R)
+	char * saveptr;
+#endif
 
+	if (!dup) 
+	        return -1;
+
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_STRTOK_R)
+	p = strtok_r (dup, ", \t", &saveptr);
+#else
+	p = strtok (dup, ", \t");
+#endif
         if (p) {
                 do {
                         if      (0 == strcmp (p, _("alert")))
@@ -149,27 +167,36 @@ static int set_prelude_severity_int (char * str, int prelude_sev)
                                 clear_and_set (prelude_sev, SH_ERR_NOTICE);
                         else if (0 == strcmp (p, _("debug")))
 	                        clear_and_set (prelude_sev, SH_ERR_ALL);
-                        else
+                        else if (0 == strcmp (p, _("info")))
+	                        clear_and_set (prelude_sev, SH_ERR_INFO);
+                        else {
+			        free (dup);
 	                        return -1;
+			}
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_STRTOK_R)
+                        p = strtok_r (NULL, ", \t", &saveptr);
+#else
                         p = strtok (NULL, ", \t");
+#endif
                 } while (p);
         }
+	free(dup);
         return 0;
 }
 
-int sh_prelude_map_info (char * str)
+int sh_prelude_map_info (const char * str)
 {
         return (set_prelude_severity_int(str,(int)IDMEF_IMPACT_SEVERITY_INFO));
 }
-int sh_prelude_map_low (char * str)
+int sh_prelude_map_low (const char * str)
 {
         return (set_prelude_severity_int(str,(int)IDMEF_IMPACT_SEVERITY_LOW));
 }
-int sh_prelude_map_medium (char * str)
+int sh_prelude_map_medium (const char * str)
 {
         return (set_prelude_severity_int(str,(int)IDMEF_IMPACT_SEVERITY_MEDIUM));
 }
-int sh_prelude_map_high (char * str)
+int sh_prelude_map_high (const char * str)
 {
         return (set_prelude_severity_int(str,(int)IDMEF_IMPACT_SEVERITY_HIGH));
 }
@@ -294,6 +321,7 @@ static int idmef_time_from_samhain(idmef_time_t **time, const char *str)
         return 0;
 }
 
+/* flawfinder: ignore *//* is part of name, not access() */
 static void get_access_info(idmef_file_access_t *access, char * mode, int pos, int mpos)
 {
         int got = 0;
@@ -302,21 +330,24 @@ static void get_access_info(idmef_file_access_t *access, char * mode, int pos, i
 
 	do {
 	        if ( mode[pos] == 'r' ) {
-	                ret = idmef_file_access_new_permission(access, &str, -1);
+			/* flawfinder: ignore *//* is part of name, not access() */
+	                ret = idmef_file_access_new_permission(access, &str, IDMEF_LIST_APPEND);
 	                if ( ret < 0 )
 	                        return;
 	                prelude_string_set_dup(str, _("read"));
 	                ++got;
 	        }
 	        else if ( mode[pos] == 'w' ) {
-	                ret = idmef_file_access_new_permission(access, &str, -1);
+			/* flawfinder: ignore *//* is part of name, not access() */
+	                ret = idmef_file_access_new_permission(access, &str, IDMEF_LIST_APPEND);
 	                if ( ret < 0 )
 	                        return;
 	                prelude_string_set_dup(str, _("write"));
 	                ++got;
 	        }
 	        else if ( mode[pos] == 'x' || mode[pos] == 's' || mode[pos] == 't') {
-	                ret = idmef_file_access_new_permission(access, &str, -1);
+			/* flawfinder: ignore *//* is part of name, not access() */
+	                ret = idmef_file_access_new_permission(access, &str, IDMEF_LIST_APPEND);
 	                if ( ret < 0 )
 	                        return;
                         
@@ -334,7 +365,8 @@ static void get_access_info(idmef_file_access_t *access, char * mode, int pos, i
 	} while (pos <= mpos);
 
 	if ( got == 0 ) {
-	        ret = idmef_file_access_new_permission(access, &str, -1);
+	        /* flawfinder: ignore *//* is part of name, not access() */
+	        ret = idmef_file_access_new_permission(access, &str, IDMEF_LIST_APPEND);
 	        if ( ret < 0 )
 	                return;
 	        prelude_string_set_dup(str, _("noAccess"));
@@ -354,12 +386,12 @@ static void get_file_infos(idmef_target_t *target, char *msg,
         idmef_inode_t *inode;
         prelude_string_t *str;
         idmef_checksum_t *checksum;
-        idmef_file_access_t *access;
+        idmef_file_access_t *access; /* flawfinder: ignore */
         idmef_user_id_t *userid;
         const char *suffix = (category == IDMEF_FILE_CATEGORY_CURRENT) ? "_new" : "_old";
 	char *mode = NULL;
                 
-        ret = idmef_target_new_file(target, &file, -1);
+        ret = idmef_target_new_file(target, &file, IDMEF_LIST_APPEND);
         if ( ret < 0  )
                 return;
         idmef_file_set_category(file, category);
@@ -371,17 +403,18 @@ static void get_file_infos(idmef_target_t *target, char *msg,
                  * including the name.
                  */
                 ret = idmef_file_new_path(file, &str);
-                if ( ret < 0 )
+                if ( ret < 0 ) {
+		        free(ptr);
                         return;
+		}
                 prelude_string_set_nodup(str, ptr);
 
                 ptr = strrchr(ptr, '/');
                 if ( ptr ) {
                         ret = idmef_file_new_name(file, &str);
-                        if ( ret < 0 )
-                                return;
-                        
-                        prelude_string_set_dup(str, ptr + 1);
+                        if ( ret == 0 ) {
+			        prelude_string_set_dup(str, ptr + 1);
+			}
                 }
         }
                 
@@ -394,140 +427,184 @@ static void get_file_infos(idmef_target_t *target, char *msg,
         ptr = get_time_value(msg, _("mtime"), suffix);
         if ( ptr ) {
                 ret = idmef_time_from_samhain(&time, ptr);
-                if ( ret < 0 )
-                        return;
-                
-                idmef_file_set_modify_time(file, time);
+                if ( ret == 0 ) {
+                        idmef_file_set_modify_time(file, time);
+		}
                 free(ptr);
         }
 
         ptr = get_time_value(msg, _("ctime"), suffix);
         if ( ptr ) {
                 ret = idmef_time_from_samhain(&time, ptr);
-                if ( ret < 0 )
-                        return;
-                
-                idmef_file_set_create_time(file, time);
+                if ( ret == 0 ) {
+                        idmef_file_set_create_time(file, time);
+		}
                 free(ptr);
         }
                 
         ptr = get_value(msg, _("inode"), suffix);
         if ( ptr ) {
                 ret = idmef_file_new_inode(file, &inode);
-                if ( ret < 0 )
-                        return;
-                
-                idmef_inode_set_number(inode, strtoul(ptr, NULL, 10));
+                if ( ret == 0 ) {
+			char * dev = get_value(msg, _("dev"), suffix);
+			if (dev) {
+			        char * q = strchr(dev, ',');
+				if (*q) {
+				         *q = '\0'; ++q;
+					 idmef_inode_set_major_device(inode, strtoul(dev, NULL, 0));
+					 idmef_inode_set_minor_device(inode, strtoul(  q, NULL, 0));
+				}
+				free(dev);
+			}
+			idmef_inode_set_number(inode, strtoul(ptr, NULL, 10));
+		}
                 free(ptr);
         }
 
         ptr = get_value(msg, _("chksum"), suffix);
         if ( ptr ) {
-                ret = idmef_file_new_checksum(file, &checksum, 0);
-                if ( ret < 0 )
-                        return;
+                ret = idmef_file_new_checksum(file, &checksum, IDMEF_LIST_APPEND);
+                if ( ret < 0 ) {
+			free(ptr);
+			goto get_mode;
+		}
 
 		hashtype = sh_tiger_get_hashtype();
 
 		if (hashtype == 0)
-                        idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_TIGER);
+			idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_TIGER);
 
-                else if (hashtype == 1)
-                        idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_SHA1);
-
-                else if (hashtype == 2)
-                        idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_MD5);
-
-                else
-                        idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_TIGER);
+		else if (hashtype == 1)
+			idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_SHA1);
+		
+		else if (hashtype == 2)
+			idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_MD5);
+		
+		else
+			idmef_checksum_set_algorithm(checksum, IDMEF_CHECKSUM_ALGORITHM_TIGER);
 
 
-                ret = idmef_checksum_new_value(checksum, &str);
-                if ( ret < 0 )
-                        return;
+		ret = idmef_checksum_new_value(checksum, &str);
+		if ( ret < 0 ) {
+			free(ptr);
+			goto get_mode;
+		}
 
 		/* will be freed on destroy()
 		 */
-                prelude_string_set_nodup(str, ptr);
-        }
+		prelude_string_set_nodup(str, ptr);
+	}
+
+ get_mode:
 
 	mode = get_value(msg, _("mode"), suffix);
         if ( mode ) {
-                ret = idmef_file_new_file_access(file, &access, -1);
+	        /* flawfinder: ignore *//* is part of name, not access() */
+                ret = idmef_file_new_file_access(file, &access, IDMEF_LIST_APPEND);
                 if ( ret < 0 )
-                        return;
+                        goto get_owner;
 
+	        /* flawfinder: ignore *//* is part of name, not access() */
                 ret = idmef_file_access_new_user_id(access, &userid);
                 if ( ret < 0 )
-                        return;
+                        goto get_owner;
                 idmef_user_id_set_type(userid, IDMEF_USER_ID_TYPE_OTHER_PRIVS);
 
+	        /* flawfinder: ignore *//* is part of name, not access() */
 		get_access_info ( access, mode, 7, 9 );
         }
+
+ get_owner:
  
         ptr = get_value(msg, _("owner"), suffix);
         if ( ptr ) {
-                struct passwd *pw;
+	        char * uid;
                 
-                ret = idmef_file_new_file_access(file, &access, 0);
-                if ( ret < 0 )
-                        return;
+	        /* flawfinder: ignore *//* is part of name, not access() */
+                ret = idmef_file_new_file_access(file, &access, IDMEF_LIST_APPEND);
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto get_group;
+		}
 
+	        /* flawfinder: ignore *//* is part of name, not access() */
                 ret = idmef_file_access_new_user_id(access, &userid);
-                if ( ret < 0 )
-                        return;
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto get_group;
+		}
                 idmef_user_id_set_type(userid, IDMEF_USER_ID_TYPE_USER_PRIVS);
                 
                 ret = idmef_user_id_new_name(userid, &str);
-                if ( ret < 0 )
-                        return;
-                
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto get_group;
+                }
                 prelude_string_set_nodup(str, ptr);
                 
-                pw = getpwnam(ptr);
-                if ( ! pw )
-                        return;
+                uid = get_value(msg, _("iowner"), suffix);
+                if ( ! uid )
+                        goto get_group;
                 
-                idmef_user_id_set_number(userid, pw->pw_uid);
+                idmef_user_id_set_number(userid, strtoul(uid, NULL, 0));
 
 		if ( mode ) {
+		        /* flawfinder: ignore *//* is part of name, not access() */
 		        get_access_info ( access, mode, 1, 3 );
 		}
+
+		free(uid);
         }
 
+ get_group:
+
         ptr = get_value(msg, _("group"), suffix);
         if ( ptr ) {
-                struct group *gr;
+                char *gid;
                 
-                ret = idmef_file_new_file_access(file, &access, -1);
-                if ( ret < 0 )
-                        return;
+	        /* flawfinder: ignore *//* is part of name, not access() */
+                ret = idmef_file_new_file_access(file, &access, IDMEF_LIST_APPEND);
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto mode_free;
+		}
+
+                ret = idmef_file_access_new_user_id(access, &userid);/* flawfinder: ignore *//* is part of name, not access() */
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto mode_free;
+		}
 
-                ret = idmef_file_access_new_user_id(access, &userid);
-                if ( ret < 0 )
-                        return;
                 idmef_user_id_set_type(userid, IDMEF_USER_ID_TYPE_GROUP_PRIVS);
                 
                 ret = idmef_user_id_new_name(userid, &str);
-                if ( ret < 0 )
-                        return;
+                if ( ret < 0 ) {
+                        free(ptr);
+                        goto mode_free;
+		}
                 
                 prelude_string_set_nodup(str, ptr);
 
-                gr = getgrnam(ptr);
-                if ( ! gr )
-                        return;
+                gid = get_value(msg, _("igroup"), suffix);
+                if ( ! gid )
+                        goto mode_free;
 
-                idmef_user_id_set_number(userid, gr->gr_gid);
+                idmef_user_id_set_number(userid, strtoul(gid, NULL, 0));
 
 		if ( mode ) {
-		        get_access_info ( access, mode, 4, 6 );
+		        get_access_info ( access, mode, 4, 6 ); /* flawfinder: ignore */
 		}
+
+		free(gid);
         }
 
+ mode_free:
+
 	if ( mode ) {
 	        free ( mode );
 	}
+
+	return;
 }
 
 
@@ -573,6 +650,13 @@ static int map_policy_to_class(char *msg, unsigned long msgid, idmef_impact_t *i
                 { MSG_KERN_SYSCALL, N_("Kernel Modified"), IDMEF_IMPACT_TYPE_OTHER },
 #endif
 
+#ifdef SH_USE_PORTCHECK
+		{ MSG_PORT_MISS, N_("Service closed"), IDMEF_IMPACT_TYPE_OTHER },
+		{ MSG_PORT_NEW, N_("Service opened"), IDMEF_IMPACT_TYPE_OTHER },
+		{ MSG_PORT_RESTART, N_("Service restarted"), IDMEF_IMPACT_TYPE_OTHER },
+		{ MSG_PORT_NEWPORT, N_("Service restarted"), IDMEF_IMPACT_TYPE_OTHER },
+#endif
+
 #ifdef SH_USE_SUIDCHK
                 { MSG_SUID_POLICY, N_("SUID/SGID File Detected"), IDMEF_IMPACT_TYPE_FILE },
 #endif
@@ -606,24 +690,302 @@ static int map_policy_to_class(char *msg, unsigned long msgid, idmef_impact_t *i
 }
 
 
+#ifdef SH_USE_PORTCHECK
+static int get_service_info(char *msg, idmef_alert_t *alert)
+{
+        int ret;
+	long port;
+	char *ptr, *new, *tmp, *ip, *srv, *protocol, *end;
+        prelude_string_t *str;
+        idmef_address_t *address;
+        idmef_node_t *node;
+	idmef_user_t *user;
+	idmef_process_t *process;
+        idmef_service_t *service;
+        idmef_source_t *source = idmef_alert_get_next_source(alert, NULL);
+        struct passwd *pw;
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+	struct passwd pwd;
+	char * buffer;
+#endif
+
+        new = sh_util_strdup(msg);
+ 
+        ptr = strstr(new, _("port: "));
+        if ( ! ptr ) {
+                sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+                                _("malformed Samhain port check message"), _("get_service_info"));
+		SH_FREE( new );
+                return -1;
+        }
+
+        ptr += 6; /* skip 'port: ', position on first byte of interface */
+        tmp = strchr(ptr, ':');
+        if ( ! tmp ) {
+                sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+                                _("malformed Samhain port check message (no port)"), _("get_service_info"));
+		SH_FREE( new );
+                return -1;
+        }
+	*tmp = '\0';
+
+	/* Get interface 
+	 */
+	ip = strdup(ptr);
+        if ( ip ) {
+                if ( ! source ) {
+                        ret = idmef_alert_new_source(alert, &source, IDMEF_LIST_APPEND);
+                        if ( ret < 0 ) {
+                                free(ip);
+				SH_FREE( new );
+                                return ret;
+                        }
+                }
+
+                ret = idmef_source_new_node(source, &node);
+                if ( ret < 0 ) {
+                        free(ip);
+			SH_FREE( new );
+                        return ret;
+                }
+                
+                ret = idmef_node_new_address(node, &address, IDMEF_LIST_APPEND);
+                if ( ret < 0 ) {
+                        free(ip);
+			SH_FREE( new );
+                        return ret;
+                }
+                
+                ret = idmef_address_new_address(address, &str);
+                if ( ret < 0 ) {
+                        free(ip);
+			SH_FREE( new );
+                        return ret;
+                }
+                
+                prelude_string_set_nodup(str, ip);
+        }
+
+	ptr = tmp;
+	++ptr;
+        tmp = strchr(ptr, '/');
+        if ( ! tmp ) {
+                sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+                                _("malformed Samhain port check message (no protocol)"), _("get_service_info"));
+		SH_FREE( new );
+                return -1;
+        }
+	*tmp = '\0';
+
+	/* Get port number
+	 */
+	port = strtol(ptr, &end, 0);
+        if ( *ptr && *end == '\0' && port >= 0 && port < 65536) {
+
+	        char * tmpw;
+
+                if ( ! source ) {
+                        ret = idmef_alert_new_source(alert, &source, IDMEF_LIST_APPEND);
+                        if ( ret < 0 ) {
+				SH_FREE( new );
+                                return ret;
+                        }
+                }
+
+                ret = idmef_source_new_service(source, &service);
+                if ( ret < 0 ) {
+			SH_FREE( new );
+                        return ret;
+                }
+
+		idmef_service_set_port(service, port);
+
+                ret = idmef_service_new_protocol(service, &str);
+                if ( ret < 0 ) {
+			SH_FREE( new );
+                        return ret;
+                }
+                
+		++tmp; 
+		if (*tmp) { 
+		        char * tmpw = tmp;
+			char tmpw_store;
+			while (*tmpw && !isblank((int) *tmpw)) ++tmpw;
+			tmpw_store = *tmpw; *tmpw = '\0';
+		        protocol = strdup(tmp);
+			*tmpw = tmpw_store;
+			prelude_string_set_nodup(str, protocol);
+		}
+
+	}
+
+	ptr = tmp;
+	++ptr;
+        ptr = strchr(ptr, '(');
+        if ( ! ptr ) {
+                sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+                                _("malformed Samhain port check message (no service)"), _("get_service_info"));
+		SH_FREE( new );
+                return -1;
+        }
+	++ptr;
+        tmp = strchr(ptr, ')');
+        if ( ! tmp ) {
+                sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+                                _("malformed Samhain port check message (service not closed)"), _("get_service_info"));
+		SH_FREE( new );
+                return -1;
+        }
+	*tmp = '\0';
+
+	/* Get service
+	 */
+	srv = strdup(ptr);
+        if ( srv ) {
+                if ( ! source ) {
+                        ret = idmef_alert_new_source(alert, &source, IDMEF_LIST_APPEND);
+                        if ( ret < 0 ) {
+                                free(srv);
+				SH_FREE( new );
+                                return ret;
+                        }
+                }
+
+		if ( ! service ) {
+                        ret = idmef_source_new_service(source, &service);
+			if ( ret < 0 ) {
+                                free(srv);
+				SH_FREE( new );
+				return ret;
+			}
+                }
+
+                ret = idmef_service_new_name(service, &str);
+                if ( ret < 0 ) {
+                        free(srv);
+			SH_FREE( new );
+                        return ret;
+                }
+                
+                prelude_string_set_nodup(str, srv);
+        }
+
+	SH_FREE( new );
+
+        ptr = get_value(msg, _("userid"), NULL);
+
+        if ( ptr ) {
+
+	        idmef_user_id_t * user_id;
+
+	        ret = idmef_source_new_user(source, &user);
+                if ( ret < 0 ) {
+                        free(ptr);
+                        return ret;
+                }
+
+                idmef_user_set_category(user, IDMEF_USER_CATEGORY_APPLICATION);
+                
+                ret = idmef_user_new_user_id(user, &user_id, IDMEF_LIST_APPEND);
+                if ( ret < 0 ) {
+                        free(ptr);
+                        return ret;
+                }
+                
+                idmef_user_id_set_type(user_id, IDMEF_USER_ID_TYPE_CURRENT_USER);
+
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+		buffer = SH_ALLOC(SH_PWBUF_SIZE);
+		sh_getpwnam_r(ptr, &pwd, buffer, SH_PWBUF_SIZE, &pw);
+#else
+		pw = sh_getpwnam(ptr);
+#endif
+                if ( pw )
+                        idmef_user_id_set_number(user_id, pw->pw_uid);
+
+                ret = idmef_user_id_new_name(user_id, &str);
+                if ( ret < 0 ) {
+                        free(ptr);
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+			SH_FREE(buffer);
+#endif
+                        return ret;
+                }
+                prelude_string_set_nodup(str, ptr);
+
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+		SH_FREE(buffer);
+#endif
+	}
+
+
+        ptr = get_value(msg, _("path"), NULL);
+        tmp = get_value(msg, _("pid"), NULL);
+
+        if ( ptr ) {
+
+                /*
+                 * In term of IDMEF, this is the full path,
+                 * including the name.
+                 */
+                ret = idmef_source_new_process(source, &process);
+                if ( ret < 0 ) {
+		        free(ptr);
+                        return ret;
+		}
+
+		ret = idmef_process_new_path(process, &str);
+                if ( ret < 0 ) {
+		        free(ptr);
+                        return ret;
+		}
+                prelude_string_set_nodup(str, ptr);
+
+                
+                if ( NULL != strrchr(ptr, '/') ) {
+                        ret = idmef_process_new_name(process, &str);
+                        if ( ret == 0 ) {
+			        ptr = strrchr(ptr, '/');
+			        prelude_string_set_dup(str, ptr + 1);
+			}
+                } else {
+		        ret = idmef_process_new_name(process, &str);
+                        if ( ret == 0 ) {
+			        prelude_string_set_dup(str, ptr);
+			}
+		}
+
+		idmef_process_set_pid(process, strtoul(tmp, NULL, 0));
+        }
+
+	if (tmp)
+	  free(tmp);
+
+	return 0;
+}
+#endif
 
 static int get_login_info(char *msg, idmef_alert_t *alert)
 {
         int ret;
-        char *ptr;
+        char *ptr, *ip;
         idmef_user_t *user;
         idmef_node_t *node;
         struct passwd *pw;
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+	struct passwd pwd;
+	char * buffer;
+#endif
         prelude_string_t *str;
         idmef_user_id_t *user_id;
         idmef_address_t *address;
         idmef_target_t *target = idmef_alert_get_next_target(alert, NULL);
         idmef_source_t *source = idmef_alert_get_next_source(alert, NULL);
           
-        ptr = get_value(msg, _("host"), NULL);
+        ip = ptr = get_value(msg, _("ip"), NULL);
         if ( ptr ) {
                 if ( ! source ) {
-                        ret = idmef_alert_new_source(alert, &source, -1);
+                        ret = idmef_alert_new_source(alert, &source, IDMEF_LIST_APPEND);
                         if ( ret < 0 ) {
                                 free(ptr);
                                 return ret;
@@ -636,32 +998,7 @@ static int get_login_info(char *msg, idmef_alert_t *alert)
                         return ret;
                 }
                 
-                ret = idmef_node_new_name(node, &str);
-                if ( ret < 0 ) {
-                        free(ptr);
-                        return ret;
-                }
-                
-                prelude_string_set_nodup(str, ptr);
-        }
-
-        ptr = get_value(msg, _("ip"), NULL);
-        if ( ptr ) {
-                if ( ! source ) {
-                        ret = idmef_alert_new_source(alert, &source, -1);
-                        if ( ret < 0 ) {
-                                free(ptr);
-                                return ret;
-                        }
-                }
-
-                ret = idmef_source_new_node(source, &node);
-                if ( ret < 0 ) {
-                        free(ptr);
-                        return ret;
-                }
-                
-                ret = idmef_node_new_address(node, &address, -1);
+                ret = idmef_node_new_address(node, &address, IDMEF_LIST_APPEND);
                 if ( ret < 0 ) {
                         free(ptr);
                         return ret;
@@ -676,6 +1013,35 @@ static int get_login_info(char *msg, idmef_alert_t *alert)
                 prelude_string_set_nodup(str, ptr);
         }
 
+        ptr = get_value(msg, _("host"), NULL);
+        if ( ptr ) {
+                if ( ip && strcmp(ptr, ip) == 0 )
+                        free(ptr);
+                else {
+                        if ( ! source ) {
+                                ret = idmef_alert_new_source(alert, &source, IDMEF_LIST_APPEND);
+                                if ( ret < 0 ) {
+                                        free(ptr);
+                                        return ret;
+                                }
+                        }
+
+                        ret = idmef_source_new_node(source, &node);
+                        if ( ret < 0 ) {
+                                free(ptr);
+                                return ret;
+                        }
+                
+                        ret = idmef_node_new_name(node, &str);
+                        if ( ret < 0 ) {
+                                free(ptr);
+                                return ret;
+                        }
+                
+                        prelude_string_set_nodup(str, ptr);
+                }
+        }
+        
         ptr = get_value(msg, _("name"), NULL);
         if ( ptr ) {
                 ret = idmef_target_new_user(target, &user);
@@ -686,7 +1052,7 @@ static int get_login_info(char *msg, idmef_alert_t *alert)
                 
                 idmef_user_set_category(user, IDMEF_USER_CATEGORY_OS_DEVICE);
                 
-                ret = idmef_user_new_user_id(user, &user_id, -1);
+                ret = idmef_user_new_user_id(user, &user_id, IDMEF_LIST_APPEND);
                 if ( ret < 0 ) {
                         free(ptr);
                         return ret;
@@ -694,13 +1060,21 @@ static int get_login_info(char *msg, idmef_alert_t *alert)
                 
                 idmef_user_id_set_type(user_id, IDMEF_USER_ID_TYPE_TARGET_USER);
                 
-                pw = getpwnam(ptr);
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+		buffer = SH_ALLOC(SH_PWBUF_SIZE);
+		sh_getpwnam_r(ptr, &pwd, buffer, SH_PWBUF_SIZE, &pw);
+#else
+		pw = sh_getpwnam(ptr);
+#endif
                 if ( pw )
                         idmef_user_id_set_number(user_id, pw->pw_uid);
 
                 ret = idmef_user_id_new_name(user_id, &str);
                 if ( ret < 0 ) {
                         free(ptr);
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+			SH_FREE(buffer);
+#endif
                         return ret;
                 }
                 prelude_string_set_nodup(str, ptr);
@@ -710,11 +1084,17 @@ static int get_login_info(char *msg, idmef_alert_t *alert)
                         ret = idmef_user_id_new_tty(user_id, &str);
                         if ( ret < 0 ) {
                                 free(ptr);
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+				SH_FREE(buffer);
+#endif
                                 return ret;
                         }
                         
                         prelude_string_set_nodup(str, ptr);
                 }
+#if defined(HAVE_PTHREAD) && defined (_POSIX_THREAD_SAFE_FUNCTIONS) && defined(HAVE_GETPWNAM_R)
+		SH_FREE(buffer);
+#endif
         }
 
         ptr = get_time_value(msg, _("time"), NULL);
@@ -749,7 +1129,7 @@ static int samhain_alert_prelude(int priority, int sh_class,
         idmef_confidence_t *confidence;
         prelude_string_t *str;
                 
-        if ( sh_class == STAMP)
+        if ( !client || sh_class == STAMP)
                 return 0;
         
         ret = idmef_message_new(&idmef);
@@ -760,7 +1140,7 @@ static int samhain_alert_prelude(int priority, int sh_class,
         if ( ret < 0 )
                 goto err;
 
-        idmef_alert_set_analyzer(alert, idmef_analyzer_ref(prelude_client_get_analyzer(client)), 0);
+        idmef_alert_set_analyzer(alert, idmef_analyzer_ref(prelude_client_get_analyzer(client)), IDMEF_LIST_PREPEND);
         
         ret = idmef_time_new_from_gettimeofday(&time);
         if ( ret < 0 )
@@ -776,7 +1156,7 @@ static int samhain_alert_prelude(int priority, int sh_class,
         if ( ret < 0 )
                 goto err;
         
-        ret = idmef_alert_new_target(alert, &target, -1);
+        ret = idmef_alert_new_target(alert, &target, IDMEF_LIST_APPEND);
         if ( ret < 0 )
                 goto err;
 
@@ -788,8 +1168,10 @@ static int samhain_alert_prelude(int priority, int sh_class,
         }
         
         if ( strstr(message, _("path=")) ) {
+#if defined(SH_WITH_CLIENT) || defined(SH_STANDALONE)
                 if ( msgid != MSG_FI_ADD && msgid != MSG_FI_ADD2 )
                         get_file_infos(target, message, IDMEF_FILE_CATEGORY_ORIGINAL);
+#endif
                 
                 get_file_infos(target, message, IDMEF_FILE_CATEGORY_CURRENT);
         }
@@ -810,6 +1192,14 @@ static int samhain_alert_prelude(int priority, int sh_class,
         if ( ret < 0 )
                 goto err;
         
+#ifdef SH_USE_PORTCHECK
+	if (msgid == MSG_PORT_MISS || msgid == MSG_PORT_NEW || msgid == MSG_PORT_RESTART || msgid == MSG_PORT_NEWPORT) {
+	        ret = get_service_info(message, alert);
+		if ( ret < 0 )
+		        goto err;
+        }
+#endif
+   
         map_policy_to_class(message, msgid, impact, str);
 
 #if 0
@@ -832,7 +1222,7 @@ static int samhain_alert_prelude(int priority, int sh_class,
 
         idmef_confidence_set_rating(confidence, IDMEF_CONFIDENCE_RATING_HIGH);
         
-        ret = idmef_alert_new_additional_data(alert, &data, -1);
+        ret = idmef_alert_new_additional_data(alert, &data, IDMEF_LIST_APPEND);
         if ( ret < 0 )
                 goto err;
 
@@ -863,7 +1253,7 @@ int sh_prelude_alert(int priority, int sh_class, char *message, long msgflags, u
         
 	(void) msgflags; /* fix compiler warning */
 
-        if ( initialized < 0 )
+        if ( initialized < 1 )
                 return -1;
         
         ret = samhain_alert_prelude(priority, sh_class, message, msgid);
@@ -878,7 +1268,7 @@ int sh_prelude_alert(int priority, int sh_class, char *message, long msgflags, u
 
 
 
-int sh_prelude_set_profile(char *arg)
+int sh_prelude_set_profile(const char *arg)
 {
         if ( profile ) {
                 free(profile);
@@ -894,7 +1284,10 @@ int sh_prelude_set_profile(char *arg)
         return 0;
 }
 
-
+/* Allow initialization of prelude; to be called
+ * after forking the daemon. Delays heartbeat
+ * start after config read until it is safe.
+ */
 void sh_prelude_reset(void)
 {
         extern void sh_error_init_prelude();
@@ -910,7 +1303,16 @@ void sh_prelude_stop(void)
 {
         if (initialized < 1)
                 return;
-        prelude_client_destroy(client, PRELUDE_CLIENT_EXIT_STATUS_SUCCESS);
+
+	if (sh.flag.isdaemon == S_TRUE)
+	        prelude_client_destroy(client, PRELUDE_CLIENT_EXIT_STATUS_FAILURE);
+	else
+	        prelude_client_destroy(client, PRELUDE_CLIENT_EXIT_STATUS_SUCCESS);
+
+	client = NULL;
+
+	prelude_deinit();
+
         initialized = 0;
         return;
 }
@@ -923,6 +1325,10 @@ int sh_prelude_init(void)
         prelude_string_t *str;
         idmef_analyzer_t *analyzer;
         prelude_client_flags_t flags;
+#ifdef SH_NOFAILOVER
+	prelude_connection_pool_t *pool;
+	prelude_connection_pool_flags_t conn_flags;
+#endif
 
 	if (ready_for_init == 0)
 	  return initialized;
@@ -931,7 +1337,7 @@ int sh_prelude_init(void)
 	  return initialized;
 
 	prelude_thread_init(NULL);
-        prelude_init(NULL, NULL);
+        prelude_init(&sh_argc_store, sh_argv_store);
 
         ret = prelude_client_new(&client, profile ? profile : _("samhain"));
         if ( ret < 0 ) {
@@ -958,6 +1364,14 @@ int sh_prelude_init(void)
 	ret = idmef_analyzer_new_version(analyzer, &str);
         prelude_string_set_dup(str, IDMEF_ANALYZER_VERSION);
         
+#ifdef SH_NOFAILOVER
+	pool = prelude_client_get_connection_pool(client);
+	conn_flags = prelude_connection_pool_get_flags(pool);
+
+	conn_flags &= ~PRELUDE_CONNECTION_POOL_FLAGS_FAILOVER;
+	prelude_connection_pool_set_flags(pool, conn_flags);
+#endif
+
         ret = prelude_client_start(client);
         if ( ret < 0 ) {
                 prelude_perror(ret, _("error starting prelude client"));
