@@ -77,6 +77,8 @@
 #include <sys/mman.h>
 #endif
 
+#define SH_REAL_SET
+
 #include "samhain.h"
 #include "sh_mem.h"
 #include "sh_error.h"
@@ -85,64 +87,130 @@
 #include "sh_tiger.h"
 #define SH_NEED_GETHOSTBYXXX
 #include "sh_static.h"
+#include "sh_pthread.h"
 
 #undef  FIL__
 #define FIL__  _("sh_tools.c")
 
 #ifdef SH_ENCRYPT
 #include "rijndael-api-fst.h"
-char * errorExplain (int err_num)
+char * errorExplain (int err_num, char * buffer, size_t len)
 {
+  char * p;
+
   if      (err_num == BAD_KEY_DIR)
-    return (_("Key direction is invalid"));
+    p = (_("Key direction is invalid"));
   else if (err_num == BAD_KEY_MAT) 
-    return (_("Key material not of correct length"));
+    p = (_("Key material not of correct length"));
   else if (err_num == BAD_KEY_INSTANCE) 
-    return (_("Key passed is not valid"));
+    p = (_("Key passed is not valid"));
   else if (err_num == BAD_CIPHER_MODE) 
-    return (_("Params struct passed to cipherInit invalid"));
+    p = (_("Params struct passed to cipherInit invalid"));
   else if (err_num == BAD_CIPHER_STATE) 
-    return (_("Cipher in wrong state"));
+    p = (_("Cipher in wrong state"));
   else if (err_num == BAD_BLOCK_LENGTH) 
-    return (_("Bad block length"));
+    p = (_("Bad block length"));
   else if (err_num == BAD_CIPHER_INSTANCE) 
-    return (_("Bad cipher instance"));
+    p = (_("Bad cipher instance"));
   else if (err_num == BAD_DATA) 
-    return (_("Data contents are invalid"));
+    p = (_("Data contents are invalid"));
   else  
-    return (_("Unknown error"));
+    p = (_("Unknown error"));
+  sl_strlcpy (buffer, p, len);
+  return buffer;
 }
 
 #endif
 
+/* --- check for an interface ---
+ */
+int sh_tools_iface_is_present(char *str)
+{
+  struct sockaddr_in sin;
+  int sd;
+
+  memset(&sin, '\0', sizeof(sin));
+  sin.sin_family = AF_INET;
+  if (inet_aton(str, &(sin.sin_addr)))
+    {
+      sin.sin_port = htons(2543);
+
+      if (-1 == (sd = socket(AF_INET, SOCK_STREAM, 0)))
+	{
+	  return 0;
+	}
+
+      if (-1 == bind(sd, (struct sockaddr *)&sin, sizeof(sin)))
+	{
+	  int retval = 0;
+
+	  /* bind() fails for access reasons, iface exists
+	   */
+	  if (errno == EACCES || errno == EADDRINUSE)
+	    retval = 1;
+	  sl_close_fd (FIL__, __LINE__, sd);
+	  return retval;
+	}
+
+      /* bind() succeeds, iface exists
+       */
+      sl_close_fd(FIL__, __LINE__, sd);
+      return 1;
+    }
+  return 0;
+}
+
 /* --- recode all \blah escapes to '=XX' format, and also code all
  *     remaining unprintable chars                                 ---
  */
+#define SH_PUT_4(p, a, b, c) (p)[0] = (a); (p)[1] = (b); (p)[2] = (c);
+  
 char * sh_tools_safe_name (const char * instr, int flag)
 {
-  unsigned char c;
+  unsigned char c, d;
   const  char * p;
   char   tmp[4];
   char * outstr;
-  int    len;
+  size_t len = 1;
   int    i = 0;
   unsigned char   val_octal = '\0';
+  static char ctable[16] = { '0', '1', '2', '3', '4', '5', '6', '7', 
+			     '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' }; 
 
   SL_ENTER(_("sh_tools_safe_name"));
 
   if (instr)
-    len = (3 * strlen(instr)) + 4;
+    {
+      len = strlen(instr);
+      if (sl_ok_muls (3, len) && sl_ok_adds ((3*len), 4))
+	{
+	  len = (3 * len) + 4;
+	  p = instr;
+	}
+      else
+	{
+	  len = 1;
+	  p   = NULL;
+	}
+    }
   else
-    len = 1;
+    {
+      p = NULL;
+    }
 
   outstr = SH_ALLOC(len);
 
   outstr[0] = '\0';
   tmp[3]    = '\0';
 
-  p = instr;
+#if !defined(SH_USE_XML)
+  (void) flag; /* fix compiler warning */
+#endif
 
-  while (p && *p)
+  if (!p)
+    goto end;
+
+  while (*p)
     {
       c = *p;
 
@@ -153,52 +221,62 @@ char * sh_tools_safe_name (const char * instr, int flag)
 	}
 
 #ifdef SH_USE_XML
-      if (flag == 1 && (*p) == '"')
-	{ 
-	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;  /* known to fit  */
-	  continue;
-	} 
-      else if (flag == 1 && (*p) == '&')
-	{ 
-	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;  /* known to fit  */
-	  continue;
-	} 
-      else if (flag == 1 && (*p) == '<') 
-	{     /* left angle       */
-	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;  /* known to fit  */
-	  continue;
-	} 
-      else if (flag == 1 && (*p) == '>') 
-	{     /* right angle      */
-	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;  /* known to fit  */
-	  continue;
+      if (flag == 1)
+	{
+	  if ((*p) == '"')
+	    { 
+	      SH_PUT_4(&outstr[i], '=', '2', '2');
+	      i+=3; ++p;
+	      continue;
+	    } 
+	  else if ((*p) == '&')
+	    { 
+	      SH_PUT_4(&outstr[i], '=', '2', '6');
+	      i+=3; ++p;
+	      continue;
+	    } 
+	  else if ((*p) == '<') 
+	    {     /* left angle       */
+	      SH_PUT_4(&outstr[i], '=', '3', 'c');
+	      i+=3; ++p;
+	      continue;
+	    } 
+	  else if ((*p) == '>') 
+	    {     /* right angle      */
+	      SH_PUT_4(&outstr[i], '=', '3', 'e');
+	      i+=3; ++p;
+	      continue;
+	    }
 	}
-#else
-      (void) flag; /* fix compiler warning */
 #endif
 
       if ( (*p) != '\\' && (*p) != '&' && (*p) != '='  && (*p) != '\'') 
         {
-	  if (c < 31 || c > 126)
+	  outstr[i] = *p; ++i;
+	  ++p;
+	      
+	  if (c < 32 || c > 126)
 	    {
-	      sprintf(&outstr[i], "=%02x", c); i+=3; ++p;/* known to fit  */
-	    }
-	  else
-	    {
-	      outstr[i] = *p;
-	      ++i; ++p;
+	      --i;
+	      d = c % 16; c = c / 16;
+	      outstr[i] = '=';       ++i;
+	      outstr[i] = ctable[c]; ++i;
+	      outstr[i] = ctable[d]; ++i;
 	    }
+
 	  continue;
 	}
       else if ((*p) == '\'')
 	{
-	  sprintf(&outstr[i], "=%02x", c);   i+=3; ++p;  /* known to fit  */
+	  SH_PUT_4(&outstr[i], '=', '2', '7');
+	  i+=3; ++p;
 	}
       else if (*p == '=')
 	{
 	  if (p[1] != '"' && p[1] != '<')
 	    { 
-	      sprintf(&outstr[i], "=%02x", c); i+=3; ++p;/* known to fit  */
+	      SH_PUT_4(&outstr[i], '=', '3', 'd');
+	      i+=3; ++p;
 	    }
 	  else
 	    { outstr[i] = *p; ++i; ++p; }
@@ -215,46 +293,47 @@ char * sh_tools_safe_name (const char * instr, int flag)
 
 	  switch (*p) {
 	  case '\\':
-	    sprintf(&outstr[i], "=%02x", c);  i+=3; ++p; /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '5', 'c');
+	    i+=3; ++p;
 	    break;
 	  case 'n':
-	    sprintf(&outstr[i], "=%02x", '\n');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', 'a');
 	    i+=3; ++p;
 	    break;
 	  case 'b':
-	    sprintf(&outstr[i], "=%02x", '\b');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', '8');
 	    i+=3; ++p;
 	    break;		       
 	  case 'r':		       
-	    sprintf(&outstr[i], "=%02x", '\r');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', 'd');
 	    i+=3; ++p;
 	    break;		       
 	  case 't':		       
-	    sprintf(&outstr[i], "=%02x", '\t');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', '9');
 	    i+=3; ++p;
 	    break;		       
 	  case 'v':		       
-	    sprintf(&outstr[i], "=%02x", '\v');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', 'b');
 	    i+=3; ++p;
 	    break;		       
 	  case 'f':		       
-	    sprintf(&outstr[i], "=%02x", '\f');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '0', 'c');
 	    i+=3; ++p;
 	    break;		       
 	  case '\'':		       
-	    sprintf(&outstr[i], "=%02x", '\'');          /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '2', '7');
 	    i+=3; ++p;
 	    break;		       
-	  case '\"':	/* also encode quoted '"' */ 	       
-	    sprintf(&outstr[i], "=%02x", '\"');          /* known to fit  */
+	  case '"':	/* also encode quoted '"' */ 	       
+	    SH_PUT_4(&outstr[i], '=', '2', '2');
 	    i+=3; ++p;
 	    break;		       
 	  case ' ':		       
-	    sprintf(&outstr[i], "=%02x", ' ');           /* known to fit  */
+	    SH_PUT_4(&outstr[i], '=', '2', '0');
 	    i+=3; ++p;
 	    break;
 	  default:
-	    if (strlen(p) < 3)
+	    if (strlen(p) < 3) /* certainly not an octal number, skip */
 	      {
 		p += strlen(p);
 	      }
@@ -264,46 +343,58 @@ char * sh_tools_safe_name (const char * instr, int flag)
 		val_octal = (unsigned char) strtoul(tmp, (char **)NULL, 8);
 		if (val_octal != '\0') { 
 		  c = val_octal;
-		  sprintf(&outstr[i], "=%02x", c);       /* known to fit  */
-		  i+=3;
+		  d = c % 16; c = c / 16;
+		  outstr[i] = '=';       ++i;
+		  outstr[i] = ctable[c]; ++i;
+		  outstr[i] = ctable[d]; ++i;
 		} 
 		p += 3;
 	      }
 	  }
 	}
-      else /* *p == '&' */
+      else if (*p == '&')
 	{
 	  ++p;
-	  if (!p || !(*p)) 
-	    break;
+	  if (!p || !(*p))
+	    {
+	      outstr[i] = '&'; ++i;
+	      break;
+	    }
 
 	  if (p[0] == 'a' && p[1] == 'm' && p[2] == 'p' && p[3] == ';')
 	    {
-	      sprintf(&outstr[i], "=%02x", '&');         /* known to fit  */
+	      SH_PUT_4(&outstr[i], '=', '2', '6');
 	      i+=3; p += 4;
 	    }
 	  else if (p[0] == 'q' && p[1] == 'u' && p[2] == 'o' && p[3] == 't' &&
 		   p[4] == ';')
 	    {
-	      sprintf(&outstr[i], "=%02x", '"');         /* known to fit  */
+	      SH_PUT_4(&outstr[i], '=', '2', '2');
 	      i+=3; p += 5;
 	    }
 	  else if (p[0] == 'l' && p[1] == 't' && p[2] == ';')
 	    {
-	      sprintf(&outstr[i], "=%02x", '<');         /* known to fit  */
+	      SH_PUT_4(&outstr[i], '=', '3', 'c');
 	      i+=3; p += 3;
 	    }
 	  else if (p[0] == 'g' && p[1] == 't' && p[2] == ';')
 	    {
-	      sprintf(&outstr[i], "=%02x", '>');         /* known to fit  */
+	      SH_PUT_4(&outstr[i], '=', '3', 'e');
 	      i+=3; p += 3;
 	    }
 	  else /* conserve the '&' */
 	    {
 	      outstr[i] = '&'; ++i;
 	    }
-	}     
+	}
+      else
+	{
+	  outstr[i] = *p; ++i;
+	  ++p;
+	}
     } /* while (p && *p) */
+
+ end:
   
   outstr[i] = '\0';
   SL_RETURN( outstr, _("sh_tools_safe_name"));
@@ -312,26 +403,28 @@ char * sh_tools_safe_name (const char * instr, int flag)
 
 /* extern int h_errno; */ 
 
-char * sh_tools_errmessage (int tellme)
+char * sh_tools_errmessage (int tellme, char * errbuf, size_t len)
 {
-
+  char * p = NULL;
 #ifdef HOST_NOT_FOUND
     if (tellme == HOST_NOT_FOUND)  
-      return _("The specified host is unknown: ");
+      p = _("The specified host is unknown: ");
 #endif
 #ifdef NO_ADDRESS
     if (tellme == NO_ADDRESS)  
-      return _("The requested name is valid but does not have an IP address: ");
+      p = _("The requested name is valid but does not have an IP address: ");
 #endif
 #ifdef NO_RECOVERY
     if (tellme == NO_RECOVERY)  
-      return _("A non-recoverable name server error occurred: ");
+      p = _("A non-recoverable name server error occurred: ");
 #endif
 #ifdef TRY_AGAIN
     if (tellme == TRY_AGAIN)  
-      return _("A temporary error occurred on an authoritative name server. The specified host is unknown: ");
+      p = _("A temporary error occurred on an authoritative name server. The specified host is unknown: ");
 #endif
-    return _("Unknown error");
+    if (!p) p =  _("Unknown error");
+    sl_strlcpy(errbuf, p, len);
+    return errbuf;
 }
 
 int is_numeric (const char * address)
@@ -404,7 +497,7 @@ void delete_cache()
       
 int DoReverseLookup = S_TRUE;
 
-int set_reverse_lookup (char * c)
+int set_reverse_lookup (const char * c)
 {
   return sh_util_flagval(c, &DoReverseLookup);
 }
@@ -419,12 +512,13 @@ int connect_port (char * address, int port,
 
   char   * host_name;
 
-  int    fd = (-1);
+  volatile int    fd = (-1);
   int    status;
-  int    fail   = 0;
+  volatile int    fail   = 0;
   int    cached = 0;
 
   int    retval;
+  char   errbuf[SH_ERRBUF_SIZE];
 
   sin_cache * check_cache = conn_cache;
 
@@ -465,7 +559,10 @@ int connect_port (char * address, int port,
       if ((unsigned long)-1  == (haddr.s_addr = inet_addr(address)))
 #endif
 	{
-      
+	  SH_MUTEX_LOCK(mutex_resolv);
+
+	  host_name = NULL;
+
 	  host_entry = sh_gethostbyname(address);
 
 	  if (host_entry == NULL || host_entry->h_addr == NULL) 
@@ -476,7 +573,7 @@ int connect_port (char * address, int port,
 #else
 	      *errnum = 666;
 #endif
-	      sl_strlcpy(errmsg, sh_tools_errmessage (*errnum), errsiz);
+	      (void) sh_tools_errmessage (*errnum, errmsg, errsiz);
 	      sl_strlcat(errmsg, address, errsiz); 
 	      fail = (-1);
 	    }
@@ -498,12 +595,7 @@ int connect_port (char * address, int port,
 		    }
 		  else
 		    {
-		      host_name = SH_ALLOC(sl_strlen(host_entry->h_name) + 1);
-		      if (sl_strlen(host_entry->h_name) > 0)
-			strcpy(host_name,                /* known to fit  */
-			       host_entry->h_name);
-		      else
-			host_name[0] = '\0';
+		      host_name = sh_util_strdup(host_entry->h_name);
 		    }
 
 		  host_entry = sh_gethostbyaddr ((char *) &sinr.sin_addr, 
@@ -517,8 +609,7 @@ int connect_port (char * address, int port,
 #else
 		      *errnum = 666;
 #endif
-		      sl_strlcpy(errmsg, 
-				 sh_tools_errmessage (*errnum), errsiz);
+		      (void) sh_tools_errmessage (*errnum, errmsg, errsiz);
 		      sl_strlcat(errmsg, 
 				 inet_ntoa (*(struct in_addr *) &(sinr.sin_addr)),
 				 errsiz); 
@@ -528,13 +619,13 @@ int connect_port (char * address, int port,
 		    {
 		      *errnum = 0;
 		      if (sl_strlen(host_entry->h_name) == 0 || 
-			  (*errnum = sl_strcmp(host_name,host_entry->h_name)) != 0)
+			  (*errnum = sl_strcasecmp(host_name,host_entry->h_name)) != 0)
 			{ 
 			  if (*errnum)
 			    sl_strlcpy(ecall, _("strcmp"), SH_MINIBUF);
 			  else
 			    sl_strlcpy(ecall, _("strlen"), SH_MINIBUF);
-			  sl_strlcpy(errmsg, _("Reverse lookup failed."), 
+			  sl_strlcpy(errmsg, _("Reverse lookup failed: "), 
 				     errsiz);
 			  sl_strlcat(errmsg, address, errsiz);
 			  sl_strlcat(errmsg, _(" vs "), errsiz);
@@ -544,10 +635,10 @@ int connect_port (char * address, int port,
 			  fail = -1;
 			}
 		    }
-
-		  SH_FREE(host_name);
 		}
 	    }
+	  SH_MUTEX_UNLOCK(mutex_resolv);
+	  if (host_name) SH_FREE(host_name);
 	}
   
       else  /* address was numeric */
@@ -593,7 +684,7 @@ int connect_port (char * address, int port,
 	status = errno;
 	sl_strlcpy(ecall, _("socket"), SH_MINIBUF);
 	*errnum = status;
-	sl_strlcpy(errmsg, sh_error_message (status), errsiz);
+	sl_strlcpy(errmsg, sh_error_message (status, errbuf, sizeof(errbuf)), errsiz);
 	sl_strlcat(errmsg, _(", address "), errsiz);
 	sl_strlcat(errmsg, address, errsiz);
       }
@@ -607,10 +698,10 @@ int connect_port (char * address, int port,
 	status = errno;
 	sl_strlcpy(ecall, _("connect"), SH_MINIBUF);
 	*errnum = status;
-	sl_strlcpy(errmsg, sh_error_message (status), errsiz);
+	sl_strlcpy(errmsg, sh_error_message (status, errbuf, sizeof(errbuf)), errsiz);
 	sl_strlcat(errmsg, _(", address "), errsiz);
 	sl_strlcat(errmsg, address, errsiz);
-	close(fd);
+	sl_close_fd(FIL__, __LINE__, fd);
 	fail = (-1); 
       }
   }
@@ -657,7 +748,7 @@ int connect_port_2 (char * address1, char * address2, int port,
   /* return retval; */
 }
 
-#if defined(HAVE_NTIME) || defined(SH_WITH_CLIENT)
+#if defined(HAVE_NTIME) || defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 static
 int sh_write_select(int type, int sockfd, 
 		    char *buf, int nbytes, 
@@ -671,12 +762,15 @@ int sh_write_select(int type, int sockfd,
   
   struct  sigaction  new_act;
   struct  sigaction  old_act;
+  char    errbuf[SH_ERRBUF_SIZE];
 
   SL_ENTER(_("sh_write_select"));
 
   /* ignore SIGPIPE (instead get EPIPE if connection is closed)
    */
   new_act.sa_handler = SIG_IGN;
+  sigemptyset( &new_act.sa_mask );         /* set an empty mask       */
+  new_act.sa_flags = 0;                    /* init sa_flags           */
   sigaction (SIGPIPE, &new_act, &old_act);
   
   FD_ZERO(&fds);
@@ -703,12 +797,15 @@ int sh_write_select(int type, int sockfd,
 		sig_raised = 2;
 		continue;
 	      }
-	    if ( errno == EINTR) /* try again */
+	    if ( errno == EINTR || errno == EINPROGRESS ) /* try again */
 	      continue;
 	    *w_error = errno;
-	    TPT(( 0, FIL__, __LINE__, _("msg=<select: %s>\n"), 
-		  sh_error_message(*w_error)));
 	    sigaction (SIGPIPE, &old_act, NULL);
+	    sh_error_message(*w_error, errbuf, sizeof(errbuf));
+	    sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, errno, MSG_E_SUBGEN,
+			     errbuf,
+			     _("sh_write_select (ws)") ); 
+	    TPT(( 0, FIL__, __LINE__, _("msg=<select: %s>\n"), errbuf ));
 	    SL_RETURN( countbytes, _("sh_write_select"));
 	  }
       }
@@ -721,12 +818,15 @@ int sh_write_select(int type, int sockfd,
 		sig_raised = 2;
 		continue;
 	      }
-	    if ( errno == EINTR ) /* try again */
+	    if ( errno == EINTR || errno == EINPROGRESS ) /* try again */
 	      continue;
 	    *w_error = errno;
-	    TPT(( 0, FIL__, __LINE__, _("msg=<select: %s>\n"), 
-		  sh_error_message(*w_error)));
 	    sigaction (SIGPIPE, &old_act, NULL);
+	    sh_error_message(*w_error, errbuf, sizeof(errbuf));
+	    sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, errno, MSG_E_SUBGEN,
+			     errbuf,
+			     _("sh_write_select (rs)") ); 
+	    TPT(( 0, FIL__, __LINE__, _("msg=<select: %s>\n"), errbuf ));
 	    SL_RETURN( countbytes, _("sh_write_select"));
 	  }
       }
@@ -777,6 +877,11 @@ int sh_write_select(int type, int sockfd,
 	  {
 	    *w_error = errno;
 	    sigaction (SIGPIPE, &old_act, NULL);
+	    sh_error_message(*w_error, errbuf, sizeof(errbuf));
+	    sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, errno, MSG_E_SUBGEN,
+			     errbuf,
+			     (type == SH_DO_WRITE) ? 
+			     _("sh_write_select (w)") : _("sh_write_select (r)")); 
 	    TPT(( 0, FIL__, __LINE__, _("msg=<count < 0>\n")));
 	    SL_RETURN( countbytes, _("sh_write_select"));
 	  }
@@ -802,42 +907,42 @@ int sh_write_select(int type, int sockfd,
 }
 #endif
 
-#if defined (SH_WITH_CLIENT)
+#if defined (SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 unsigned long write_port (int sockfd, char *buf, unsigned long nbytes, 
 			  int * w_error, int timeout)
 {
   unsigned long bytes;
-  char errmsg[256];
 
   SL_ENTER(_("write_port"));
 
   bytes = sh_write_select(SH_DO_WRITE, sockfd, buf, nbytes, w_error, timeout);
   if (*w_error != 0)
     {
-      sl_strlcpy(errmsg, sh_error_message (*w_error), sizeof(errmsg));
+      char errbuf[SH_ERRBUF_SIZE];
       sh_error_handle((-1), FIL__, __LINE__, *w_error, MSG_TCP_NETRP, 
-		      errmsg, (long) sockfd, _("write_port"));
+		      sh_error_message (*w_error, errbuf, sizeof(errbuf)),
+		      (long) sockfd, _("write_port"));
     }
   SL_RETURN( bytes, _("write_port"));
 }
 #endif
 
-#if defined(HAVE_NTIME) || defined(SH_WITH_CLIENT)
+#if defined(HAVE_NTIME) || defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 
 unsigned long read_port (int sockfd, char *buf, unsigned long nbytes, 
 	       int * w_error, int timeout)
 {
   unsigned long bytes;
-  char errmsg[256];
 
   SL_ENTER(_("read_port"));
 
   bytes = sh_write_select(SH_DO_READ, sockfd, buf, nbytes, w_error, timeout);
   if (*w_error != 0)
     {
-      sl_strlcpy(errmsg, sh_error_message (*w_error), sizeof(errmsg));
+      char errbuf[SH_ERRBUF_SIZE];
       sh_error_handle((-1), FIL__, __LINE__, *w_error, MSG_TCP_NETRP, 
-		      errmsg, (long) sockfd, _("read_port"));
+		      sh_error_message (*w_error, errbuf, sizeof(errbuf)),
+		      (long) sockfd, _("read_port"));
     }
   SL_RETURN( bytes, _("read_port"));
 }
@@ -858,7 +963,7 @@ int check_request_nerr (char * have, char * need)
 }
 #endif
 
-#if defined (SH_WITH_CLIENT)
+#if defined (SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 
 int check_request (char * have, char * need)
 {
@@ -912,7 +1017,7 @@ int check_request_s (char * have, char * need, char * clt)
 }
 #endif
 
-#if defined (SH_WITH_CLIENT)
+#if defined (SH_WITH_CLIENT) || defined (SH_WITH_SERVER)
 
 void get_header (unsigned char * head, unsigned long * bytes, char * u)
 {
@@ -992,6 +1097,11 @@ void put_header (unsigned char * head, int protocol,
  * full_size <= 8192; payload_size <= 8176 (511*16); msg_size <= 8128 (508*16)
  * (msg_size = payload_size - key_len = payload_size - 48)
  */ 
+
+/* 
+ * only SH_V2_FULLSIZE is used, and only once
+ */
+#if 0
 #ifdef SH_WITH_SERVER
 #define SH_V2_FULLSIZE  240
 #define SH_V2_PAYLOAD   224
@@ -1001,6 +1111,8 @@ void put_header (unsigned char * head, int protocol,
 #define SH_V2_PAYLOAD  1008
 #define SH_V2_MESSAGE   960
 #endif
+#endif
+#define SH_V2_FULLSIZE 1024
 
 #ifdef SH_ENCRYPT
 #include "rijndael-api-fst.h"
@@ -1051,17 +1163,17 @@ void sh_tools_show_header (unsigned char * head, char sign)
  * last 4 bytes of outgoing header are set to dummy value
  */
 char * sh_tools_makePack (unsigned char * header, 
-			  char * payload, int payload_size,
+			  char * payload, unsigned long payload_size,
 			  keyInstance * keyInstE)
 {
   UINT32 rpad[3];
   unsigned char   head[16];
   double epad;
-  int    i_epad = 0;
-  int    i_blk = payload_size / 16;
-  int    i_blkmax = SH_V2_FULLSIZE / 16;
-  int    pads = 0;
-  int    full_size;
+  unsigned long    i_epad = 0;
+  unsigned long    i_blk = payload_size / 16;
+  unsigned long    i_blkmax = SH_V2_FULLSIZE / 16;
+  unsigned long    pads = 0;
+  size_t full_size;
   char * full_ret;
 
   char                  * p;
@@ -1071,6 +1183,7 @@ char * sh_tools_makePack (unsigned char * header,
   cipherInstance          cipherInst;
   int                     err_num;
   int                     blkfac;
+  int                     oflow = 0;
 
   /* 
      SL_REQUIRE (i_blk*16 == payload_size, _("payload_size % 16 != 0"));
@@ -1081,11 +1194,11 @@ char * sh_tools_makePack (unsigned char * header,
 #endif
   /* random_pad
    */
-  rpad[1] = taus_get (&(skey->rng0[0]), &(skey->rng1[0]), &(skey->rng2[0]));
+  rpad[1] = taus_get ();
   memcpy (head,      &rpad[1],    4);
-  rpad[0] = taus_get (&(skey->rng0[0]), &(skey->rng1[0]), &(skey->rng2[0]));
+  rpad[0] = taus_get ();
   memcpy (&head[4],  &rpad[0],    4);
-  rpad[2] = taus_get (&(skey->rng0[0]), &(skey->rng1[0]), &(skey->rng2[0]));
+  rpad[2] = taus_get ();
   memcpy (&head[8],  &rpad[2],    4);
 
   /* protocol
@@ -1107,24 +1220,36 @@ char * sh_tools_makePack (unsigned char * header,
 #ifdef DEBUG_EN2
     fprintf(stderr, "PAD1 <%d> <%f>\n", pads, epad);
 #endif
-    i_epad = (int) (pads * epad);
+    i_epad = (unsigned long) (pads * epad);
 #ifdef DEBUG_EN2
     fprintf(stderr, "PAD2 <%d> <%d>\n", i_epad, (i_epad*16));
 #endif
   }
 
-  full_size = 
-  /* head     */ 16 + 
-  /* payload  */ (i_blk*16) + /* payload_size + */ 
-  /* pad      */ (i_epad * 16);
+  full_size =  16;                        /* head     */
+  if (sl_ok_muls(i_blk, 16) && sl_ok_adds(full_size, (i_blk*16)))
+    full_size =  full_size + (i_blk*16);  /* payload  */
+  else
+    oflow = 1;
+  if (sl_ok_adds(full_size, (i_epad*16)))
+    full_size =  full_size + (i_epad*16); /* pad      */
+  else
+    i_epad = 0;
+
+  if (oflow)
+    {
+      sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+		      _("integer overflow"), 
+		      _("sh_tools_makePack"));
+    }
 
   full_ret = SH_ALLOC(full_size);
   memcpy(full_ret,                   head,    16);
-  if (payload != NULL)
+  if (payload != NULL && !oflow)
     {
       memcpy(&full_ret[16],              payload, payload_size);
     }
-  if ((i_blk*16) > payload_size) 
+  if ((i_blk*16) > payload_size && !oflow) 
     {
 #ifdef DEBUG_EN2
       fprintf(stderr, "SEN2 <%d>\n", (i_blk*16) - payload_size);
@@ -1150,8 +1275,9 @@ char * sh_tools_makePack (unsigned char * header,
   
   if (err_num < 0) 
     {
+      char expbuf[SH_ERRBUF_SIZE];
       sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-		      errorExplain(err_num), 
+		      errorExplain(err_num, expbuf, sizeof(expbuf)), 
 		      _("sh_tools_makePack: cipherInit"));
     }
   for (j = 0; j < blkfac; ++j)
@@ -1161,8 +1287,9 @@ char * sh_tools_makePack (unsigned char * header,
 			     inBlock, 128 * BNUM, outBlock);
       if (err_num < 0)
 	{
+	  char expbuf[SH_ERRBUF_SIZE];
 	  sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-			  errorExplain(err_num), 
+			  errorExplain(err_num, expbuf, sizeof(expbuf)), 
 			  _("sh_tools_makePack: blockEncrypt"));
 	}
       memcpy(p, outBlock, B_SIZ);
@@ -1189,6 +1316,7 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
   cipherInstance          cipherInst;
   int                     err_num;
   int                     blkfac;
+  char expbuf[SH_ERRBUF_SIZE];
 
   msg_size = (256 * (unsigned int)header[1] + (unsigned int)header[2]);
 #ifdef DEBUG_EN2
@@ -1208,7 +1336,7 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
   if (err_num < 0) 
     {
       sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-		      errorExplain(err_num), 
+		      errorExplain(err_num, expbuf, sizeof(expbuf)), 
 		      _("sh_tools_revertPack: cipherInit"));
     }
   for (j = 0; j < blkfac; ++j)
@@ -1219,7 +1347,7 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
       if (err_num < 0)
 	{
 	  sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
-			  errorExplain(err_num), 
+			  errorExplain(err_num, expbuf, sizeof(expbuf)), 
 			  _("sh_tools_revertPack: blockDecrypt"));
 	}
       memcpy(p, outBlock, B_SIZ);
@@ -1268,10 +1396,11 @@ char * sh_tools_revertPack (unsigned char * header, char * message,
 int sh_tools_hash_add(char * key, char * buf, int buflen)
 {
   char         * theSig;
+  char sigbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("sh_tools_hash_add"));
 
-  theSig = sh_util_siggen (key, buf, buflen);
+  theSig = sh_util_siggen (key, buf, buflen, sigbuf, sizeof(sigbuf));
   sl_strlcat(buf, theSig, buflen + KEY_LEN + 1);
       
   SL_RETURN((0), _("sh_tools_hash_add"));
@@ -1285,10 +1414,11 @@ int sh_tools_hash_vfy(char * key, char * buf, int buflen)
   char           hash[KEY_LEN+1];
   register int   i;
   char         * theSig;
+  char sigbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("sh_tools_hash_vfy"));
 
-  theSig = sh_util_siggen (key, buf, buflen);
+  theSig = sh_util_siggen (key, buf, buflen, sigbuf, sizeof(sigbuf));
   sl_strlcpy(hash, theSig, KEY_LEN+1);
 
   for (i = 0; i < KEY_LEN; ++i)
@@ -1309,10 +1439,11 @@ int sh_tools_hash_vfy(char * key, char * buf, int buflen)
 char * hash_me (char * key, char * buf,   int buflen)
 {
   char           hash[KEY_LEN+1];
-  char         * temp;
+  char         * temp = NULL;
   register int   i;
   int            total = 0;
   char         * theSig;
+  char sigbuf[KEYBUF_SIZE];
 
 
   SL_ENTER(_("hash_me"));
@@ -1326,24 +1457,32 @@ char * hash_me (char * key, char * buf,   int buflen)
   ASSERT_RET((key != NULL), _("key != NULL"), (NULL));
   ASSERT_RET((buflen >= 0), _("buflen >= 0"), (NULL));
 
-  theSig = sh_util_siggen (key, buf, buflen);
+  theSig = sh_util_siggen (key, buf, buflen, sigbuf, sizeof(sigbuf));
   sl_strlcpy(hash, theSig, KEY_LEN+1);
 
+  if (sl_ok_adds(buflen, KEY_LEN))
+    {
+      total = KEY_LEN + buflen;
+      temp  = SH_ALLOC (total);
 
-  total = KEY_LEN + buflen;
-  temp  = SH_ALLOC (total);
-
-  for (i = 0; i < KEY_LEN; ++i)
-    temp[i] = hash[i];
-
-  for (i = 0; i < buflen; ++i)
-    temp[i+KEY_LEN] = buf[i];
+      for (i = 0; i < KEY_LEN; ++i)
+	temp[i] = hash[i];
 
+      for (i = 0; i < buflen; ++i)
+	temp[i+KEY_LEN] = buf[i];
+    }
+  else
+    {
+      sh_error_handle((-1), FIL__, __LINE__, -1, MSG_E_SUBGEN,
+		      _("integer overflow"), 
+		      _("hash_me"));
+      temp = sh_util_strdup(buf);
+    }
   SL_RETURN(temp, _("hash_me"));
 }
 #endif
 
-#if defined (SH_WITH_CLIENT)
+#if defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER)
 
 /* verify the checksum of a buffer; checksum comes first
  */
@@ -1353,6 +1492,7 @@ int hash_check(char * key,
   char           hash[KEY_LEN+1];
   register int   i;
   char         * theSig;
+  char sigbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("hash_check"));
 
@@ -1360,7 +1500,8 @@ int hash_check(char * key,
   fprintf(stderr, "hash_check <%s> <%d>\n", 
 	  (key == NULL) ? "NULL" : key, buflen);
 #endif
-  theSig = sh_util_siggen (key, &buf[KEY_LEN], buflen-KEY_LEN);
+  theSig = sh_util_siggen (key, &buf[KEY_LEN], buflen-KEY_LEN,
+			   sigbuf, sizeof(sigbuf));
   sl_strlcpy(hash, theSig, KEY_LEN+1);
       
   for (i = 0; i < KEY_LEN; ++i)
@@ -1378,18 +1519,20 @@ int hash_check(char * key,
 char * get_client_conf_file (char * peer, unsigned long * length)
 {
   char * ret;
-  int    size, status;
+  int    status;
   struct stat buf;
   char * base;
+  size_t size;
 
   SL_ENTER(_("get_client_conf_file"));
 
-  size = sl_strlen(DEFAULT_DATAROOT);
-  base = SH_ALLOC(size + 1);
-  sl_strlcpy(base, DEFAULT_DATAROOT, size + 1); 
+  base = sh_util_strdup(DEFAULT_DATAROOT);
 
-  size = sl_strlen(base) + sl_strlen(peer) + 5;
-  ++size;
+  size = sl_strlen(base);
+  if (sl_ok_adds(size, sl_strlen(peer)))
+      size += sl_strlen(peer);
+  if (sl_ok_adds(size, 6))
+    size += 6;
 
   ret = SH_ALLOC(size);
   sl_strlcpy(ret, base, size);
@@ -1416,6 +1559,7 @@ char * get_client_conf_file (char * peer, unsigned long * length)
 		    (long) sh.effective.uid, ret);
 
   SH_FREE(base);
+  SH_FREE(ret);
   *length=0;
   SL_RETURN(NULL, _("get_client_conf_file"));
 
@@ -1435,20 +1579,21 @@ char * get_client_conf_file (char * peer, unsigned long * length)
 char * get_client_data_file (char * peer, unsigned long * length)
 {
   char * ret;
-  int    size, status;
+  int    status;
   struct stat buf;
 
   char * base;
+  size_t size;
 
   SL_ENTER(_("get_client_data_file"));
 
-  size = sl_strlen(DEFAULT_DATAROOT);
-  base = SH_ALLOC(size + 1);
-  sl_strlcpy(base, DEFAULT_DATAROOT, size + 1); 
+  base = sh_util_strdup(DEFAULT_DATAROOT);
 
-  size = sl_strlen(base) + sl_strlen(peer) + 7;
-
-  ++size;
+  size = sl_strlen(base);
+  if (sl_ok_adds(size, sl_strlen(peer)))
+      size += sl_strlen(peer);
+  if (sl_ok_adds(size, 8))
+    size += 8;
 
   ret = SH_ALLOC(size);
   sl_strlcpy(ret, base, size);
@@ -1478,6 +1623,7 @@ char * get_client_data_file (char * peer, unsigned long * length)
 
   *length = 0;
   SH_FREE(base);
+  SH_FREE(ret);
   SL_RETURN(NULL, _("get_client_data_file"));
 
  lab1_end:
@@ -1495,7 +1641,7 @@ char * get_client_data_file (char * peer, unsigned long * length)
 }
 #endif
 
-#if defined(SH_WITH_CLIENT) || defined(SH_STEALTH) || defined(WITH_GPG) || defined(WITH_PGP)
+#if defined(SH_WITH_CLIENT) || defined(SH_WITH_SERVER) || defined(SH_STEALTH) || defined(WITH_GPG) || defined(WITH_PGP)
 
 /* --------- secure temporary file ------------ */
 
@@ -1508,23 +1654,17 @@ SL_TICKET open_tmp ()
   int           error;
   int           status = BAD;
   char        * my_tmp_dir;
-  int           len;
+  char hashbuf[KEYBUF_SIZE];
 
   SL_ENTER(_("open_tmp"));
 
 #if defined(SH_TMPDIR)
-  len        = sl_strlen(SH_TMPDIR) + 1;
-  my_tmp_dir = SH_ALLOC(len);
-  sl_strlcpy(my_tmp_dir, SH_TMPDIR, len); 
+  my_tmp_dir = sh_util_strdup(SH_TMPDIR); 
 #else
 #if defined(SH_WITH_SERVER)
-  len        = sl_strlen(DEFAULT_LOGDIR) + 1;
-  my_tmp_dir = SH_ALLOC(len);
-  sl_strlcpy(my_tmp_dir, DEFAULT_LOGDIR, len); 
+  my_tmp_dir = sh_util_strdup(DEFAULT_LOGDIR); 
 #else
-  len        = sl_strlen(sh.effective.home) + 1;
-  my_tmp_dir = SH_ALLOC(len);
-  sl_strlcpy(my_tmp_dir, sh.effective.home, len);
+  my_tmp_dir = sh_util_strdup(sh.effective.home);
 #endif 
 #endif
 
@@ -1544,15 +1684,17 @@ SL_TICKET open_tmp ()
 
     /* create random filename in effective users home directory
      */
-    ticks = taus_get (&(skey->rng0[0]), &(skey->rng1[0]), &(skey->rng2[0]));
+    ticks = taus_get ();
     if (my_tmp_dir[0] == '/' && my_tmp_dir[1] == '\0')
       file = sh_util_strconcat (my_tmp_dir, 
-				sh_tiger_hash( (char *) &ticks, TIGER_DATA, 4),
+				sh_tiger_hash( (char *) &ticks, TIGER_DATA, 4,
+					       hashbuf, sizeof(hashbuf)),
 				NULL);
     else
       file = sh_util_strconcat (my_tmp_dir, 
 				"/", 
-				sh_tiger_hash( (char *) &ticks, TIGER_DATA, 4),
+				sh_tiger_hash( (char *) &ticks, TIGER_DATA, 4,
+					       hashbuf, sizeof(hashbuf)),
 				NULL);
 
     /* check whether it already exists (paranoia)
@@ -1567,6 +1709,8 @@ SL_TICKET open_tmp ()
       {
 	SH_FREE (file);
 	SH_FREE(my_tmp_dir);
+	sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, status, MSG_E_SUBGEN, 
+			_("Error (lstat) while opening temporary file"), _("open_tmp"));
 	TPT(( 0, FIL__, __LINE__, _("msg=<Unexpected error %d>\n"), error));
 	SL_RETURN((-1), _("open_tmp"));
       }
@@ -1592,7 +1736,7 @@ SL_TICKET open_tmp ()
 
   } while (status == BAD);
 
-  fd = sl_open_safe_rdwr (file, SL_YESPRIV);
+  fd = sl_open_safe_rdwr (FIL__, __LINE__, file, SL_YESPRIV);
   if (SL_ISERROR(fd))
     {
       sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, fd, MSG_E_SUBGEN, 
@@ -1605,7 +1749,9 @@ SL_TICKET open_tmp ()
   SH_FREE (file);
   SH_FREE(my_tmp_dir);
 
-  sl_unlink(fd); 
+  if (!SL_ISERROR(fd)) {
+    sl_unlink(fd);
+  } 
 
   if (!SL_ISERROR(fd))
     SL_RETURN((fd), _("open_tmp"));
