@@ -78,7 +78,10 @@ typedef pid_t __pid_t;
 #include "sh_error.h"
 #include "sh_modules.h"
 #include "sh_utmp.h"
+#include "sh_pthread.h"
+#include "sh_inotify.h"
 
+SH_MUTEX_EXTERN(mutex_thread_nolog);
 
 #ifdef TM_IN_SYS_TIME
 #include <sys/time.h>
@@ -184,6 +187,8 @@ typedef struct log_user {
 static char   terminated_line[UT_HOSTSIZE]; 
 #endif
 
+static char * mode_path[] = { _PATH_WTMP, _PATH_WTMP, _PATH_UTMP };
+
 static struct SH_UTMP_S save_utmp;
 
 static void sh_utmp_logout_morechecks(struct log_user   * user);
@@ -224,14 +229,17 @@ sh_rconf sh_utmp_table[] = {
   },
 };
 
-int sh_utmp_null()
+static void set_defaults(void)
 {
-  return 0;
+  ShUtmpLoginSolo    = SH_ERR_INFO;
+  ShUtmpLoginMulti   = SH_ERR_WARN;
+  ShUtmpLogout       = SH_ERR_INFO;
+  ShUtmpActive       = S_TRUE;
+  ShUtmpInterval     = 300;
+  return;
 }
 
 
-
-
 #if defined (HAVE_SETUTENT) && defined (USE_SETUTENT)
 
 #ifdef HAVE_UTMPX_H
@@ -277,7 +285,7 @@ static void sh_utmp_utmpname(const char * str)
   SL_ENTER(_("sh_utmp_utmpname"));
   if (sh_utmpfile != NULL)
     {
-      (void) fclose (sh_utmpfile);
+      (void) sl_fclose (FIL__, __LINE__, sh_utmpfile);
       sh_utmpfile = NULL;
     }
 
@@ -299,8 +307,10 @@ static void sh_utmp_setutent(void)
 
   if (sh_utmpfile == NULL) 
     {
+      SH_MUTEX_LOCK(mutex_thread_nolog);
       fd = (int) aud_open (FIL__, __LINE__, SL_NOPRIV, 
 			   sh_utmppath, O_RDONLY, 0);
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
       if (fd >= 0)
 	{
 	  sh_utmpfile = fdopen(fd, "r");
@@ -312,8 +322,10 @@ static void sh_utmp_setutent(void)
       if (sh_utmpfile == NULL) 
 	{
 	  error = errno;
+	  SH_MUTEX_LOCK(mutex_thread_nolog);
 	  sh_error_handle ((-1), FIL__, __LINE__, error, MSG_E_ACCESS,
 			   (long) sh.real.uid, sh_utmppath);
+	  SH_MUTEX_UNLOCK(mutex_thread_nolog);
 	  SL_RET0(_("sh_utmp_setutent"));
 	}
     }
@@ -330,7 +342,8 @@ static void sh_utmp_setutent(void)
 static void sh_utmp_endutent(void)
 {
   SL_ENTER(_("sh_utmp_endutent"));
-  (void) fclose(sh_utmpfile);
+  if (NULL != sh_utmpfile)
+    (void) sl_fclose(FIL__, __LINE__, sh_utmpfile);
   sh_utmpfile = NULL;
   SL_RET0(_("sh_utmp_endutent"));
 }
@@ -438,7 +451,7 @@ static char * my_inet_ntoa(struct in_addr in)
   memcpy (&b, &bar[1], 1);
   memcpy (&c, &bar[2], 1);
   memcpy (&d, &bar[3], 1);
-  sprintf(foo, "%d.%d.%d.%d",                          /* known to fit  */
+  sprintf(foo, _("%d.%d.%d.%d"),                        /* known to fit  */
 	  (int) a, (int) b, (int) c, (int) d);
   return foo;
 }
@@ -453,15 +466,16 @@ static char * my_inet_ntoa(struct in_addr in)
 
 static struct log_user   * userlist   = NULL;
 static time_t  lastcheck;
+static int     init_done = 0;
 
 /*************
  *
  * module init
  *
  *************/
-int sh_utmp_init ()
+
+static int sh_utmp_init_internal (void)
 {
-  static int done = 0;
 
   SL_ENTER(_("sh_utmp_init"));
   if (ShUtmpActive == BAD)
@@ -469,7 +483,7 @@ int sh_utmp_init ()
 
   /* do not re-initialize after a re-configuration
    */
-  if (done == 1) {
+  if (init_done == 1) {
     SL_RETURN( (0), _("sh_utmp_init"));
   }
   lastcheck  = time (NULL);
@@ -477,17 +491,38 @@ int sh_utmp_init ()
   memset (&save_utmp, 0, sizeof(struct SH_UTMP_S));
   sh_utmp_check_internal (2); /* current logins */
   sh_utmp_check_internal (0);
-  done = 1;
+  init_done = 1;
   SL_RETURN( (0), _("sh_utmp_init"));
 }
 
+int sh_utmp_init (struct mod_type * arg)
+{
+  if (ShUtmpActive == BAD)
+    return SH_MOD_FAILED;
+#ifdef HAVE_PTHREAD
+  if (arg != NULL && arg->initval < 0 && 
+      (sh.flag.isdaemon == S_TRUE || sh.flag.loop == S_TRUE))
+    {
+      if (0 == sh_pthread_create(sh_threaded_module_run, (void *)arg))
+	return SH_MOD_THREAD;
+      else
+	return SH_MOD_FAILED;
+    }
+#endif
+  return sh_utmp_init_internal();
+}
+
 /*************
  *
  * module cleanup
  *
  *************/
 #ifdef HAVE_UTTYPE
-static int sh_utmp_login_clean();
+static int sh_utmp_login_clean(void);
+#endif
+
+#if defined(HAVE_PTHREAD)
+static sh_watches inotify_watch;
 #endif
 
 int sh_utmp_end ()
@@ -506,10 +541,26 @@ int sh_utmp_end ()
 #ifdef HAVE_UTTYPE
   (void) sh_utmp_login_clean();
 #endif
+  /* Reset the flag, such that the module
+   * can be re-enabled.
+   */
+  set_defaults();
+  init_done          = 0;
+
+  sh_inotify_remove(&inotify_watch);
+
   SL_RETURN( (0), _("sh_utmp_end"));
 }
 
 
+int sh_utmp_reconf()
+{
+  set_defaults();
+  sh_inotify_remove(&inotify_watch);
+  return 0;
+}
+
+
 /*************
  *
  * module timer
@@ -517,12 +568,40 @@ int sh_utmp_end ()
  *************/
 int sh_utmp_timer (time_t tcurrent)
 {
+#if !defined(HAVE_PTHREAD)
+  retry_msleep(1, 0);
+
   if ((time_t) (tcurrent - lastcheck) >= ShUtmpInterval)
     {
       lastcheck  = tcurrent;
       return (-1);
     }
   return 0;
+#else
+  int errnum = 0;
+  
+  if ( (sh.flag.isdaemon == S_TRUE || sh.flag.loop == S_TRUE) &&
+       sh.flag.checkSum != SH_CHECK_INIT )
+    {
+      sh_inotify_wait_for_change(mode_path[1], &inotify_watch, 
+				 &errnum, ShUtmpInterval);
+    }
+  
+  lastcheck  = tcurrent;
+
+  if (SH_INOTIFY_ERROR(errnum))
+    {
+      char ebuf[SH_ERRBUF_SIZE];
+
+      SH_MUTEX_LOCK(mutex_thread_nolog);
+      sh_error_message(errnum, ebuf, sizeof(ebuf));
+      sh_error_handle (SH_ERR_WARN, FIL__, __LINE__, errnum, MSG_E_SUBGEN,
+		       ebuf,
+		       _("sh_utmp_timer") );
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);    
+    }
+  return -1;
+#endif
 }
 
 /*************
@@ -533,7 +612,16 @@ int sh_utmp_timer (time_t tcurrent)
 int sh_utmp_check ()
 {
   SL_ENTER(_("sh_utmp_check"));
+  if (ShUtmpActive == BAD)
+    {
+#if defined(HAVE_PTHREAD)
+      sh_inotify_remove(&inotify_watch);
+#endif
+      SL_RETURN( (-1), _("sh_utmp_check"));
+    }
+  SH_MUTEX_LOCK(mutex_thread_nolog);
   sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_UT_CHECK);
+  SH_MUTEX_UNLOCK(mutex_thread_nolog);
   sh_utmp_check_internal (1);
 
   SL_RETURN(0, _("sh_utmp_check"));
@@ -545,7 +633,7 @@ int sh_utmp_check ()
  *
  *************/
 
-int sh_utmp_set_login_solo  (char * c)
+int sh_utmp_set_login_solo  (const char * c)
 {
   int retval;
   char tmp[32];
@@ -553,11 +641,13 @@ int sh_utmp_set_login_solo  (char * c)
   SL_ENTER(_("sh_utmp_set_login_solo"));
   tmp[0] = '='; tmp[1] = '\0';
   (void) sl_strlcat (tmp, c, 32);
+  SH_MUTEX_LOCK(mutex_thread_nolog);
   retval = sh_error_set_level (tmp, &ShUtmpLoginSolo);
+  SH_MUTEX_UNLOCK(mutex_thread_nolog);
   SL_RETURN(retval, _("sh_utmp_set_login_solo"));
 }
 
-int sh_utmp_set_login_multi (char * c)
+int sh_utmp_set_login_multi (const char * c)
 {
   int retval;
   char tmp[32];
@@ -565,11 +655,13 @@ int sh_utmp_set_login_multi (char * c)
   SL_ENTER(_("sh_utmp_set_login_multi"));
   tmp[0] = '='; tmp[1] = '\0';
   (void) sl_strlcat (tmp, c, 32);
+  SH_MUTEX_LOCK(mutex_thread_nolog);
   retval = sh_error_set_level (tmp, &ShUtmpLoginMulti);
+  SH_MUTEX_UNLOCK(mutex_thread_nolog);
   SL_RETURN(retval, _("sh_utmp_set_login_multi"));
 }
 
-int sh_utmp_set_logout_good (char * c)
+int sh_utmp_set_logout_good (const char * c)
 {
   int retval;
   char tmp[32];
@@ -577,31 +669,32 @@ int sh_utmp_set_logout_good (char * c)
   SL_ENTER(_("sh_utmp_set_logout_good"));
   tmp[0] = '='; tmp[1] = '\0';
   (void) sl_strlcat (tmp, c, 32);
+  SH_MUTEX_LOCK(mutex_thread_nolog);
   retval = sh_error_set_level (tmp, &ShUtmpLogout);
+  SH_MUTEX_UNLOCK(mutex_thread_nolog);
   SL_RETURN(retval, _("sh_utmp_set_logout_good"));
 }
 
-int sh_utmp_set_login_timer (char * c)
+int sh_utmp_set_login_timer (const char * c)
 {
-  int retval = 0;
   long val;
 
   SL_ENTER(_("sh_utmp_set_login_timer"));
   val = strtol (c, (char **)NULL, 10);
   if (val <= 0)
     {
+      SH_MUTEX_LOCK(mutex_thread_nolog);
       sh_error_handle ((-1), FIL__, __LINE__, EINVAL, MSG_EINVALS,
 		       _("utmp timer"), c);
-      retval = -1;
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
+      SL_RETURN((-1), _("sh_utmp_set_login_timer"));
     }
 
-  val = (val <= 0 ? 60 : val);
-
   ShUtmpInterval = (time_t) val;
   SL_RETURN(0, _("sh_utmp_set_login_timer"));
 }
 
-int sh_utmp_set_login_activate (char * c)
+int sh_utmp_set_login_activate (const char * c)
 {
   int i;
   SL_ENTER(_("sh_utmp_set_login_activate"));
@@ -618,7 +711,7 @@ struct login_ct {
 
 static struct login_ct * login_ct_list = NULL;
 
-static int sh_utmp_login_clean()
+static int sh_utmp_login_clean(void)
 {
   struct login_ct * list = login_ct_list;
   struct login_ct * old;
@@ -697,28 +790,33 @@ static int sh_utmp_login_r(char * str)
  *    - link device.ut_record -> log_record
  *    - link user.ut_record   -> log_record
  */
+
 #ifdef HAVE_UTTYPE  
 static int sh_utmp_is_virtual (char * in_utline, char * in_uthost)
 {
-  if (in_utline != NULL)
-    {
-      if (0 == sl_strncmp(in_utline, _("ttyp"), 4))
-	{ return 0; }
-      else if (0 == sl_strncmp(in_utline, _("ttyq"), 4))
-	{ return 0; }
-    }
-  if (in_uthost != NULL   && 
+
+  if (in_uthost != NULL   &&
+      in_utline != NULL   &&
       in_uthost[0] == ':' && 
       in_uthost[1] == '0' && 
-      in_uthost[2] == '\0')
+      0 == sl_strncmp(in_utline, _("pts/"), 4))
     {
-      /* return 0; */
-      return 1;  /* actually does not seem portable, not even among linuxes */
+      return 1;
     }
-  return 1;
+
+  return 0;
 }
 #endif
 
+/* These variables are not used anywhere. They only exist
+ * to assign &userold, &user to them, which keeps gcc from
+ * putting them into a register, and avoids the 'clobbered
+ * by longjmp' warning. And no, 'volatile' proved insufficient.
+ */
+static void * sh_dummy_userold = NULL;
+static void * sh_dummy_user    = NULL;
+
+
 static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 {
   struct log_user   * user     = userlist;
@@ -729,11 +827,17 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 
   char   ttt[TIM_MAX];
 #ifdef HAVE_UTTYPE
-  int    status;
+  volatile int    status;
 #endif
 
   SL_ENTER(_("sh_utmp_addlogin"));
 
+  /* Take the address to keep gcc from putting them into registers. 
+   * Avoids the 'clobbered by longjmp' warning. 
+   */
+  sh_dummy_userold = (void*) &userold;
+  sh_dummy_user    = (void*) &user;
+
   if (ut->ut_line[0] == '\0')
     SL_RET0(_("sh_utmp_addlogin"));
 
@@ -752,7 +856,7 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
    */
   while (user != NULL) 
     {
-      if (0 == sl_strncmp(user->ut_tty, ut->ut_line, UT_LINESIZE) ) 
+      if (0 == sl_strncmp((char*)(user->ut_tty), ut->ut_line, UT_LINESIZE) ) 
 	break;
       userold = user;
       user = user->next;
@@ -775,18 +879,18 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 	{
 	  user = SH_ALLOC(sizeof(struct log_user));
 	  user->next       = userlist;
-	  userlist         = user;
+	  userlist         = (struct log_user *) user;
 	}
-      (void) sl_strlcpy(user->ut_tty,  ut->ut_line, UT_LINESIZE+1);
-      (void) sl_strlcpy(user->name,    ut->ut_name, UT_NAMESIZE+1);
+      (void) sl_strlcpy((char*)(user->ut_tty),  ut->ut_line, UT_LINESIZE+1);
+      (void) sl_strlcpy((char*)(user->name),    ut->ut_name, UT_NAMESIZE+1);
 #ifdef HAVE_UTHOST
-      (void) sl_strlcpy(user->ut_host, ut->ut_host, UT_HOSTSIZE+1);
+      (void) sl_strlcpy((char*)(user->ut_host), ut->ut_host, UT_HOSTSIZE+1);
 #else
       user->ut_host[0] = '\0';
 #endif
 #ifdef HAVE_UTADDR
       /*@-type@*//* ut_addr does exist !!! */
-      (void) sl_strlcpy(user->ut_ship, 
+      (void) sl_strlcpy((char*)(user->ut_ship), 
 			my_inet_ntoa(*(struct in_addr*)&(ut->ut_addr)), 16);
       /*@+type@*/
 #endif
@@ -796,8 +900,9 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
           || 0 == sl_strncmp(ut->ut_line, _("ttyp"), 4) /* in virt. console  */
           || 0 == sl_strncmp(ut->ut_line, _("ttyq"), 4) /* in virt. console  */
 	  ) {
-	status = sh_utmp_login_a(user->name);
-	(void) sl_strlcpy(ttt, sh_unix_time (user->time), TIM_MAX);
+	status = sh_utmp_login_a((char*)user->name);
+	SH_MUTEX_LOCK(mutex_thread_nolog);
+	(void) sh_unix_time (user->time, ttt, TIM_MAX);
 	sh_error_handle( ShUtmpLoginSolo, FIL__, __LINE__, 0,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
 			 MSG_UT_LG1X,
@@ -817,11 +922,13 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 			 ttt,
 			 status
 			 );
+	SH_MUTEX_UNLOCK(mutex_thread_nolog);
       } else
-	if (0 != sh_utmp_is_virtual(ut->ut_line, user->ut_host))
+	if (0 == sh_utmp_is_virtual(ut->ut_line, (char*)user->ut_host))
 	  {       
-	    status = sh_utmp_login_a(user->name);
-	    (void) sl_strlcpy(ttt, sh_unix_time (user->time), TIM_MAX);
+	    status = sh_utmp_login_a((char*)user->name);
+	    SH_MUTEX_LOCK(mutex_thread_nolog);
+	    (void) sh_unix_time (user->time, ttt, TIM_MAX);
 	    sh_error_handle( ShUtmpLoginMulti, FIL__, __LINE__, 0,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
 			     MSG_UT_LG2X,
@@ -841,6 +948,7 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 			     ttt,
 			     status
 			     );
+	    SH_MUTEX_UNLOCK(mutex_thread_nolog);
 	  }
       
       sh_utmp_login_morechecks(ut);
@@ -855,44 +963,53 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
     {
       if (user != NULL)
 	{
-	  status = sh_utmp_login_r(user->name);
-	  /* sl_strlcpy(ttt, sh_unix_time (user->time), TIM_MAX); */
-	  (void) sl_strlcpy(ttt, sh_unix_time (ut->ut_time), TIM_MAX);
-	  sh_error_handle( ShUtmpLogout, FIL__, __LINE__, 0,
+#if defined(__linux__)
+	  if (0 == sh_utmp_is_virtual(ut->ut_line, (char*)user->ut_host)) {
+#endif
+	    status = sh_utmp_login_r((char*)user->name);
+	    SH_MUTEX_LOCK(mutex_thread_nolog);
+	    (void) sh_unix_time (ut->ut_time, ttt, TIM_MAX);
+	    sh_error_handle( ShUtmpLogout, FIL__, __LINE__, 0,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
-			   MSG_UT_LG3X,
+			     MSG_UT_LG3X,
 #elif defined(HAVE_UTHOST)
-			   MSG_UT_LG3A,
+			     MSG_UT_LG3A,
 #else
-			   MSG_UT_LG3B,
+			     MSG_UT_LG3B,
 #endif
-			   user->name,
-			   user->ut_tty,
+			     user->name,
+			     user->ut_tty,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
-			   user->ut_host,
-			   user->ut_ship,
+			     user->ut_host,
+			     user->ut_ship,
 #elif defined(HAVE_UTHOST)
-			   user->ut_host,
+			     user->ut_host,
+#endif
+			     ttt,
+			     status
+			     );
+	    SH_MUTEX_UNLOCK(mutex_thread_nolog);
+	    userold->next = user->next;
+	    if (user == userlist)
+	      userlist = user->next;
+	    sh_utmp_logout_morechecks((struct log_user *)user);
+	    SH_FREE((struct log_user *)user);
+	    user = NULL;
+#if defined(__linux__)
+	  }
 #endif
-			   ttt,
-			   status
-			   );
-	  userold->next = user->next;
-	  if (user == userlist)
-	    userlist = user->next;
-	  sh_utmp_logout_morechecks(user);
-	  SH_FREE(user);
-	  user = NULL;
 	}
       else
 	{
 	  (void) sl_strlcpy(terminated_line, ut->ut_line, UT_HOSTSIZE);
-	  (void) sl_strlcpy(ttt, sh_unix_time (ut->ut_time), TIM_MAX);
+	  SH_MUTEX_LOCK(mutex_thread_nolog);
+	  (void) sh_unix_time (ut->ut_time, ttt, TIM_MAX);
 	  sh_error_handle( ShUtmpLogout, FIL__, __LINE__, 0,
 			   MSG_UT_LG3C,
 			   terminated_line,
 			   ttt, 0
 			   );
+	  SH_MUTEX_UNLOCK(mutex_thread_nolog);
 	}
       SL_RET0(_("sh_utmp_addlogin"));
     }
@@ -918,9 +1035,8 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
       user->next       = userlist;
       userlist         = user;
 
-      sl_strlcpy(ttt, sh_unix_time (user->time), TIM_MAX);
-
-
+      SH_MUTEX_LOCK(mutex_thread_nolog);
+      (void) sh_unix_time (user->time, ttt, TIM_MAX);
       sh_error_handle( ShUtmpLoginSolo, FIL__, __LINE__, 0,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
 		       MSG_UT_LG1X,
@@ -940,13 +1056,13 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 		       ttt,
 		       1
 		       );
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
       sh_utmp_login_morechecks(ut);
     }
   else  /* probably a logout */
     {
-      sl_strlcpy(ttt, sh_unix_time (ut->ut_time), TIM_MAX);
-      /* sl_strlcpy(ttt, sh_unix_time (user->time), TIM_MAX); */
-
+      SH_MUTEX_LOCK(mutex_thread_nolog);
+      (void) sh_unix_time (ut->ut_time, ttt, TIM_MAX);
       sh_error_handle( ShUtmpLogout, FIL__, __LINE__, 0,
 #if defined(HAVE_UTHOST) && defined(HAVE_UTADDR)
 		       MSG_UT_LG2X,
@@ -966,6 +1082,7 @@ static void sh_utmp_addlogin (struct SH_UTMP_S * ut)
 		       ttt,
 		       1
 		       );
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
       sh_utmp_logout_morechecks(user);
       userold->next = user->next;
       if (user == userlist)       /* inserted Apr 4, 2004 */
@@ -982,24 +1099,29 @@ static time_t        lastmod  = 0;
 static off_t         lastsize = 0;
 static unsigned long lastread = 0;
 
-static char * mode_path[] = { _PATH_WTMP, _PATH_WTMP, _PATH_UTMP };
-
 static void sh_utmp_check_internal (int mode)
 {
   struct stat   buf;
   int           error;
   struct SH_UTMP_S * ut;
-  unsigned long this_read = 0;
+  unsigned long this_read;
+  int           val_retry;
 
   SL_ENTER(_("sh_utmp_check_internal"));
 
   /* error if no access
    */
-  if (0 != retry_lstat(FIL__, __LINE__, mode_path[mode], &buf)) 
+  do {
+    val_retry = /*@-unrecog@*/lstat ( mode_path[mode], &buf)/*@+unrecog@*/;
+  } while (val_retry < 0 && errno == EINTR);
+
+  if (0 != val_retry) 
     {
       error = errno;
+      SH_MUTEX_LOCK(mutex_thread_nolog);
       sh_error_handle((-1), FIL__, __LINE__, error, MSG_E_ACCESS,
 		      (long) sh.real.uid, mode_path[mode]);
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
       SL_RET0(_("sh_utmp_check_internal"));
     }
 
@@ -1019,8 +1141,10 @@ static void sh_utmp_check_internal (int mode)
    */
   if (/*@-usedef@*/buf.st_size < lastsize/*@+usedef@*/ && mode < 2) 
     { 
+      SH_MUTEX_LOCK(mutex_thread_nolog);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_UT_ROT,
 		      mode_path[mode]);
+      SH_MUTEX_UNLOCK(mutex_thread_nolog);
       lastread = 0;
 #ifndef USE_SETUTENT
       sh_utmp_feed_forward = 0L;
@@ -1040,6 +1164,8 @@ static void sh_utmp_check_internal (int mode)
    * feed forward if initializing
    * we need to do this here
    */
+  this_read = 0;
+
   if (mode < 2)
     {
       while (this_read < lastread) {
@@ -1062,12 +1188,6 @@ static void sh_utmp_check_internal (int mode)
 #endif
 		      ))
       sh_utmp_addlogin (ut);
-    /*************************************************
-    printf("%8s | %10s | %10s | %3d %5d | %16s | %ld\n", 
-	   ut->ut_name, ut->ut_id, ut->ut_line, 
-	   (int) ut->ut_type, (int) ut->ut_pid, 
-	   ut->ut_host, ut->ut_time);
-    ***************************************************/
     ++this_read;
   }
 
